%% Reset MATLAB
clc
close all
clear all  %#ok<CLALL>

%% Constants
L=2; % Number of time-synchronized cells.
U=1; % Number of users per cell.
r_m=1000; % Cell radius (m).
d_U_BS=800; % Distance from a user to its Base Station (m).

SNR_dB=0; % Cell edge SNR.
tao=5; % Known pilot symbols.
kappa=2; % Unknown pilot symbols.

sigman2_aux_ini=1; % Noise variance.

%%% Capacity %%%
rho_dB=0;
rho=10^(rho_dB/10);
Z_dl=0;
phi_dl=1/(rho);
limit_delta_diff=1e-3;

% %%%% 1.9 GHz NLOS, 3GPP model %%%%
% scenario=3;
% f=1.9e9; % Carrier frequency.
% SP=20; % Number of subpaths per time cluster.
% TC=6; % Number of time clusters.
% SL=6; % Number of spatial AOA lobes.
% sigma_theta_AOA=5*pi/180; % Standard deviation to generate the multipaths' AOAs
% user_pos_fixed=0;
%
% %%%% 28 GHz NLOS, Rappaport model %%%%
% scenario=1;
% f=28e9; % Carrier frequency.
% SP=16; % Number of subpaths per time cluster. SP ~ DU[1,30]
% TC=4; % Number of time clusters. TC ~ DU[1,6]
% SL=2; % Number of spatial AOA lobes. E[SL] = 1.6
% sigma_theta_AOA=9*pi/180; % Standard deviation to generate the multipaths' AOAs
% n_pl=3.4; % Path loss exponent
% user_pos_fixed=0;

% %%%% 73 GHz NLOS, Rappaport model %%%%
% scenario=2;
% f=73e9; % Carrier frequency.
% SP=16; % Number of subpaths per time cluster. SP ~ DU[1,30]
% TC=4; % Number of time clusters. TC ~ DU[1,6]
% SL=2; % Number of spatial AOA lobes. E[SL] = 1.5
% sigma_theta_AOA=7*pi/180; % Standard deviation to generate the multipaths' AOAs
% n_pl=3.3; % Path loss exponent
% user_pos_fixed=0;

%%%% Simple model, Static Model %%%%
scenario=4;
f=2e9; % Carrier frequency.
SP=50; % Number of subpaths per time cluster.
TC=1; % Number of time clusters.
SL=1; % Number of spatial AOA lobes.
sigma_theta_AOA=10*pi/180;
gamma=3; % Path-loss exponent.
user_pos_fixed=0;

c=3e8; % Speed of light.
lambda=c/f; % Signal wavelength.
D=0.5*lambda; % Antenna spacing at the base station.

cell_edge_dist_rep=3; % Number of cellular network realizations.
MC_rep=10; % Number of Monte Carlo iterations.

error_vs_std_dev_AOA=0; % Estimation Error vs. Standard Deviation of AOA.
error_vs_ant_num=1; % Estimation Error vs. Number of antennas.
rate_vs_ant_num=0; % Per-cell Rate vs. Number of antennas.

sigma_AOA_ini=pi/180*10; % Standard deviation of the Gaussian AOA distribution.
M_ini=10; % Number of base station antennas.

plot_network=0;
comp_cap=0;
comp_cap_theo=0;
comp_sim=1;
noise_var_est=1;
comp_rank=0;
comp_coor=0;
cmp_known_channel=0;
same_Ptx=0;
Ptx=1e24;
R_ideal_aux=0;
R_bessel_series=50;
R_estimation_rep=1000;

different_sigman2=0;

pilot_mod=1;
if pilot_mod==1
    M_cons=2;
    psk_active=1;
    if psk_active==1
        modulator = comm.PSKModulator(M_cons);
        modulator.PhaseOffset=pi/4;
    end
    qam_active=0;
    if qam_active==1
        modulator = comm.RectangularQAMModulator(M_cons);
        modulator.NormalizationMethod='Average power';
    end
    ideal_cons_points=constellation(modulator);
end

random_AOA=0; % If '0', AOA=user position and SL y TC =1;
if random_AOA==0
    SL=1;
    TC=1;
end

AS_active=1;
if AS_active==0
    SL=TC;
    SP=1;
end
AOA_dist='Gaussian';

if noise_var_est==1
    sigman2_initial_guess=0.1; % Initial guess for the Newton-Raphson method.
    err_est_sigman2_ML=1e-3; % Error for the Newton-Raphson method.
end

if (error_vs_std_dev_AOA==1 && error_vs_ant_num==0 && rate_vs_ant_num==0)||...
        (error_vs_std_dev_AOA==0 && error_vs_ant_num==0 && rate_vs_ant_num==0)||...
        (error_vs_std_dev_AOA==0 && error_vs_ant_num==1 && rate_vs_ant_num==0)||...
        (error_vs_std_dev_AOA==0 && error_vs_ant_num==0 && rate_vs_ant_num==1)||...
        (error_vs_std_dev_AOA==1 && error_vs_ant_num==0 && rate_vs_ant_num==0)||...
        (error_vs_std_dev_AOA==0 && error_vs_ant_num==1 && rate_vs_ant_num==1)
    
else
    
    fprintf('The combination of figures is not allowed.\n')
    return
    
end

SNR_vec=-20:20:20;
sweep_f=zeros(20,length(M_ini),length(SNR_vec));
% sigma_theta_AOA_vec=(1:1:60)*pi/180;
% sweep_f=zeros(20,length(sigma_theta_AOA_vec));

for cell_edge_dist_aux=1:cell_edge_dist_rep % Cellular network realizations.
    
    sweep_aux=1;
    sweep=zeros(20,length(M_ini),length(SNR_vec));
    %     sweep=zeros(20,length(sigma_theta_AOA_vec));
    
    mean_AOA_ini=zeros(U,L);
    if user_pos_fixed==1
        for u=1:U
            mean_AOA_ini(u,:)=(pi/180)*(360/U)*u*ones(1,L); % Mean of the Uniform and Gaussian AOA distributions (L cells, U users).
        end
    else
        mean_AOA_ini=2*pi*rand(U,L);
    end
    
    if random_AOA==1
        mean_AOA=zeros(U,SL,L,L);
        for aux_L=1:L
            for aux_L2=1:L
                for aux_U=1:U
                    for aux_SL=1:SL
                        if (scenario~=4)
                            if scenario==3
                                SL_min=0;
                                SL_max=2*pi;
                            else
                                SL_min=2*pi*(aux_SL-1)/SL;
                                SL_max=2*pi*(aux_SL)/SL;
                            end
                            mean_AOA(aux_U,aux_SL,aux_L2,aux_L)=SL_min + (SL_max-SL_min)*rand(1,1);
                        else
                            mean_AOA(aux_U,aux_SL,aux_L2,aux_L)=(aux_SL)*pi/SL;
                        end
                    end
                end
            end
        end
    end
    
    %     for sigma_theta_AOA=sigma_theta_AOA_vec
    for SNR_dB=SNR_vec
        
        %% Initialization of performance metrics
        if error_vs_std_dev_AOA==1
            
            %%%% Estimation Error, Standard Deviation of AOA %%%%
            deviation_max=length(sigma_AOA_ini);
            sigma_AOA_plot=zeros(1,deviation_max);
            SNR_sim=zeros(1,deviation_max); % Average SNR obtained through simulations.
            S_sim=zeros(1,deviation_max); % Average Signal power obtained through simulations.
            N_sim=zeros(1,deviation_max); % Average Noise power obtained through simulations.
            
            err_LS=zeros(1,deviation_max); % Normalized channel estimation error for LS estimator.
            err_LS_num=zeros(1,deviation_max);
            err_LS_den=zeros(1,deviation_max);
            err_LS_no_int=zeros(1,deviation_max); % Normalized channel estimation error for LS estimator without interference.
            err_LS_no_int_num=zeros(1,deviation_max);
            err_LS_no_int_den=zeros(1,deviation_max);
            err_CB=zeros(1,deviation_max); % Normalized channel estimation error for CB estimator.
            err_CB_num=zeros(1,deviation_max);
            err_CB_den=zeros(1,deviation_max);
            err_CB_no_int=zeros(1,deviation_max); % Normalized channel estimation error for CB estimator without interference.
            err_CB_no_int_num=zeros(1,deviation_max);
            err_CB_no_int_den=zeros(1,deviation_max);
            err_CPA=zeros(1,deviation_max); % Normalized channel estimation error for CPA estimator.
            err_CPA_num=zeros(1,deviation_max);
            err_CPA_den=zeros(1,deviation_max);
            err_CPA_noise_ML=zeros(1,deviation_max); % Normalized channel estimation error for CPA estimator (with noise variance estimation (ML) ).
            err_CPA_noise_ML_num=zeros(1,deviation_max);
            err_CPA_noise_ML_den=zeros(1,deviation_max);
            err_CPA_noise_MM=zeros(1,deviation_max); % Normalized channel estimation error for CPA estimator (with noise variance estimation (MM) ).
            err_CPA_noise_MM_num=zeros(1,deviation_max);
            err_CPA_noise_MM_den=zeros(1,deviation_max);
            err_CPA_noise_NU_1dB=zeros(1,deviation_max);
            err_CPA_noise_NU_1dB_num=zeros(1,deviation_max);
            err_CPA_noise_NU_1dB_den=zeros(1,deviation_max);
            err_sigman2_ML=zeros(1,deviation_max);
            err_sigman2_ML_num=zeros(1,deviation_max);
            err_sigman2_ML_den=zeros(1,deviation_max);
            err_sigman2_MM=zeros(1,deviation_max);
            err_sigman2_MM_num=zeros(1,deviation_max);
            err_sigman2_MM_den=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML_num=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML_den=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML_alt=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML_num_alt=zeros(1,deviation_max);
            err_sigman2_unk_pilot_ML_den_alt=zeros(1,deviation_max);
            err_sigman2_unk_pilot_MM=zeros(1,deviation_max);
            err_sigman2_unk_pilot_MM_num=zeros(1,deviation_max);
            err_sigman2_unk_pilot_MM_den=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML_num=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML_den=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML_alt=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML_num_alt=zeros(1,deviation_max);
            err_sigman2_mix_pilot_ML_den_alt=zeros(1,deviation_max);
            err_sigman2_mix_pilot_MM=zeros(1,deviation_max);
            err_sigman2_mix_pilot_MM_num=zeros(1,deviation_max);
            err_sigman2_mix_pilot_MM_den=zeros(1,deviation_max);
            noise_est_vec_MM=zeros(1,cell_edge_dist_rep*MC_rep*L);
            noise_est_vec_ML=zeros(1,cell_edge_dist_rep*MC_rep*L);
            noise_est_vec_MM_aux=1;
            noise_est_vec_ML_aux=1;
            crlb=zeros(1,deviation_max);
            crlb_unk_num=zeros(1,deviation_max);
            crlb_unk_den=zeros(1,deviation_max);
            crlb_k=zeros(1,deviation_max);
            crlb_k_num=zeros(1,deviation_max);
            crlb_k_den=zeros(1,deviation_max);
            crlb_unk_pilot=zeros(M_ini(end),deviation_max);
            crlb_unk_pilot_num=zeros(M_ini(end),deviation_max);
            crlb_unk_pilot_den=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot_num=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot_den=zeros(M_ini(end),deviation_max);
            crlb_unk_pilot_alt=zeros(M_ini(end),deviation_max);
            crlb_unk_pilot_num_alt=zeros(M_ini(end),deviation_max);
            crlb_unk_pilot_den_alt=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot_alt=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot_num_alt=zeros(M_ini(end),deviation_max);
            crlb_k_unk_pilot_den_alt=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot_num=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot_den=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot_num=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot_den=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot_alt=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot_num_alt=zeros(M_ini(end),deviation_max);
            crlb_mix_pilot_den_alt=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot_alt=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot_num_alt=zeros(M_ini(end),deviation_max);
            crlb_k_mix_pilot_den_alt=zeros(M_ini(end),deviation_max);
            sigman2_sum_crlb=zeros(1,deviation_max);
            MC_aux_crlb_unk=zeros(1,deviation_max);
            MC_aux_crlb_k_unk=zeros(1,deviation_max);
            MC_aux_crlb_mix=zeros(1,deviation_max);
            MC_aux_crlb_k_mix=zeros(1,deviation_max);
            
        end
        
        aux_sigma_AOA=0;
        for sigma_AOA=sigma_AOA_ini % Standard Deviation of AOA "for loop".
            
            aux_sigma_AOA=aux_sigma_AOA+1;
            sigma_AOA_plot(aux_sigma_AOA)=sigma_AOA;
            
            %% Initialization of performance metrics
            if error_vs_ant_num==1 || rate_vs_ant_num==1
                
                M_max=length(M_ini);
                M_plot=zeros(1,M_max);
                SNR_sim=zeros(1,M_max); % Average SNR obtained through simulations.
                S_sim=zeros(1,M_max); % Average Signal power obtained through simulations.
                N_sim=zeros(1,M_max); % Average Noise power obtained through simulations.
                rank_R=zeros(2,M_max);
                mse_R=zeros(1,M_max);
                
                if error_vs_ant_num==1
                    
                    %%%% Estimation Error, Number of antennas %%%%
                    err_LS=zeros(1,M_max); % Normalized channel estimation error for LS estimator.
                    err_LS_num=zeros(1,M_max);
                    err_LS_den=zeros(1,M_max);
                    err_LS_no_int=zeros(1,M_max); % Normalized channel estimation error for LS estimator without interference.
                    err_LS_no_int_num=zeros(1,M_max);
                    err_LS_no_int_den=zeros(1,M_max);
                    err_CB=zeros(1,M_max); % Normalized channel estimation error for CB estimator.
                    err_CB_num=zeros(1,M_max);
                    err_CB_den=zeros(1,M_max);
                    err_CB_no_int=zeros(1,M_max); % Normalized channel estimation error for CB estimator without interference.
                    err_CB_no_int_num=zeros(1,M_max);
                    err_CB_no_int_den=zeros(1,M_max);
                    err_CPA=zeros(1,M_max); % Normalized channel estimation error for CPA estimator.
                    err_CPA_num=zeros(1,M_max);
                    err_CPA_den=zeros(1,M_max);
                    err_CPA_noise_ML=zeros(1,M_max); % Normalized channel estimation error for CPA estimator (with noise variance estimation (ML) ).
                    err_CPA_noise_ML_num=zeros(1,M_max);
                    err_CPA_noise_ML_den=zeros(1,M_max);
                    err_CPA_noise_MM=zeros(1,M_max); % Normalized channel estimation error for CPA estimator (with noise variance estimation (MM) ).
                    err_CPA_noise_MM_num=zeros(1,M_max);
                    err_CPA_noise_MM_den=zeros(1,M_max);
                    err_CPA_noise_NU_1dB=zeros(1,M_max);
                    err_CPA_noise_NU_1dB_num=zeros(1,M_max);
                    err_CPA_noise_NU_1dB_den=zeros(1,M_max);
                    err_sigman2_ML=zeros(1,M_max);
                    err_sigman2_ML_num=zeros(1,M_max);
                    err_sigman2_ML_den=zeros(1,M_max);
                    err_sigman2_MM=zeros(1,M_max);
                    err_sigman2_MM_num=zeros(1,M_max);
                    err_sigman2_MM_den=zeros(1,M_max);
                    err_sigman2_LS=zeros(1,M_max);
                    err_sigman2_LS_num=zeros(1,M_max);
                    err_sigman2_LS_den=zeros(1,M_max);
                    err_sigman2_LS_PSD=zeros(1,M_max);
                    err_sigman2_LS_PSD_num=zeros(1,M_max);
                    err_sigman2_LS_PSD_den=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML_num=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML_den=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML_alt=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML_num_alt=zeros(1,M_max);
                    err_sigman2_unk_pilot_ML_den_alt=zeros(1,M_max);
                    err_sigman2_unk_pilot_MM=zeros(1,M_max);
                    err_sigman2_unk_pilot_MM_num=zeros(1,M_max);
                    err_sigman2_unk_pilot_MM_den=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML_num=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML_den=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML_alt=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML_num_alt=zeros(1,M_max);
                    err_sigman2_mix_pilot_ML_den_alt=zeros(1,M_max);
                    err_sigman2_mix_pilot_MM=zeros(1,M_max);
                    err_sigman2_mix_pilot_MM_num=zeros(1,M_max);
                    err_sigman2_mix_pilot_MM_den=zeros(1,M_max);
                    noise_est_vec_MM=zeros(1,cell_edge_dist_rep*MC_rep*L);
                    noise_est_vec_ML=zeros(1,cell_edge_dist_rep*MC_rep*L);
                    noise_est_vec_MM_aux=1;
                    noise_est_vec_ML_aux=1;
                    crlb=zeros(1,M_max);
                    crlb_unk_num=zeros(1,M_max);
                    crlb_unk_den=zeros(1,M_max);
                    crlb_k=zeros(1,M_max);
                    crlb_k_num=zeros(1,M_max);
                    crlb_k_den=zeros(1,M_max);
                    crlb_unk_pilot=zeros(M_ini(end),M_max);
                    crlb_unk_pilot_num=zeros(M_ini(end),M_max);
                    crlb_unk_pilot_den=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot_num=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot_den=zeros(M_ini(end),M_max);
                    crlb_unk_pilot_alt=zeros(M_ini(end),M_max);
                    crlb_unk_pilot_num_alt=zeros(M_ini(end),M_max);
                    crlb_unk_pilot_den_alt=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot_alt=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot_num_alt=zeros(M_ini(end),M_max);
                    crlb_k_unk_pilot_den_alt=zeros(M_ini(end),M_max);
                    crlb_mix_pilot=zeros(M_ini(end),M_max);
                    crlb_mix_pilot_num=zeros(M_ini(end),M_max);
                    crlb_mix_pilot_den=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot_num=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot_den=zeros(M_ini(end),M_max);
                    crlb_mix_pilot_alt=zeros(M_ini(end),M_max);
                    crlb_mix_pilot_num_alt=zeros(M_ini(end),M_max);
                    crlb_mix_pilot_den_alt=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot_alt=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot_num_alt=zeros(M_ini(end),M_max);
                    crlb_k_mix_pilot_den_alt=zeros(M_ini(end),M_max);
                    sigman2_sum_crlb=zeros(1,M_max);
                    MC_aux_crlb=zeros(1,M_max);
                    pilot_sym_aux=1;
                    pilot_sym=zeros(tao,cell_edge_dist_rep*MC_rep);
                    conv_vec=zeros(2,MC_rep);
                    MC_aux_crlb_unk=zeros(1,M_max);
                    MC_aux_crlb_k_unk=zeros(1,M_max);
                    MC_aux_crlb_mix=zeros(1,M_max);
                    MC_aux_crlb_k_mix=zeros(1,M_max);
                    
                end
                
                if rate_vs_ant_num==1
                    
                    %%%% Per-cell Rate, Number of antennas %%%%
                    R_BF_empirical_P_S=zeros(MC_rep,M_max,U,1);
                    R_BF_empirical_P_I2=zeros(MC_rep,M_max,U,L,U,1);
                    lambda_BF_aux=zeros(MC_rep,L,M_max,1);
                    R_BF_empirical=zeros(1,M_max);
                    R_BF_empirical_coor_user=zeros(U,M_max);
                    R_BF_theoretical=zeros(1,M_max);
                    R_BF_theoretical_coor_user=zeros(U,M_max);
                    
                    R_RZF_empirical_P_S=zeros(MC_rep,M_max,U,1);
                    R_RZF_empirical_P_I2=zeros(MC_rep,M_max,U,L,U,1);
                    lambda_RZF_aux=zeros(MC_rep,L,M_max,1);
                    R_RZF_empirical=zeros(1,M_max);
                    R_RZF_empirical_coor_user=zeros(U,M_max);
                    R_RZF_theoretical=zeros(1,M_max);
                    R_RZF_theoretical_coor_user=zeros(U,M_max);
                    N_SINR=zeros(U,M_max);
                    S_SINR=zeros(U,M_max);
                    Sp_SINR=zeros(U,M_max);
                    I_SINR=zeros(U,M_max);
                    N_SINR_mean=zeros(1,M_max);
                    S_SINR_mean=zeros(1,M_max);
                    Sp_SINR_mean=zeros(1,M_max);
                    I_SINR_mean=zeros(1,M_max);
                    
                    N_SINR_emp=zeros(U,M_max);
                    S_SINR_emp=zeros(U,M_max);
                    Sp_SINR_emp=zeros(U,M_max);
                    I_SINR_emp=zeros(U,M_max);
                    N_SINR_mean_emp=zeros(1,M_max);
                    S_SINR_mean_emp=zeros(1,M_max);
                    Sp_SINR_mean_emp=zeros(1,M_max);
                    I_SINR_mean_emp=zeros(1,M_max);
                    
                end
            end
            
            if error_vs_std_dev_AOA==1
                fprintf('Sigma_AOA=%d, Realization %d\n',sigma_AOA*180/pi,cell_edge_dist_aux)
            end
            
            if plot_network==1
                
                %% Network representation %%
                figure;
                
                %%% Hexagons %%%
                x_hex=[r_m r_m*cos(60*pi/180) -r_m*cos(60*pi/180) -r_m -r_m*cos(60*pi/180) r_m*cos(60*pi/180) r_m];
                y_hex=[0 r_m*sin(60*pi/180) r_m*sin(60*pi/180) 0 -r_m*sin(60*pi/180) -r_m*sin(60*pi/180) 0];
                
                plot(x_hex,y_hex,'k'); % Cell #1;
                hold on
                plot(x_hex+(r_m+r_m*sin(30*pi/180)),y_hex+r_m*cos(30*pi/180),'k'); % Cell #2;
                if L==7
                    plot(x_hex,y_hex+(sqrt(3)*r_m),'k'); % Cell #3
                    plot(x_hex-(r_m+r_m*sin(30*pi/180)),y_hex+r_m*cos(30*pi/180),'k'); % Cell #4
                    plot(x_hex-(r_m+r_m*sin(30*pi/180)),y_hex-r_m*cos(30*pi/180),'k'); % Cell #5
                    plot(x_hex,y_hex-(sqrt(3)*r_m),'k'); % Cell #6
                    plot(x_hex+(r_m+r_m*sin(30*pi/180)),y_hex-r_m*cos(30*pi/180),'k'); % Cell #7
                end
                
                %%% Cell edge %%%
                t = linspace(0,2*pi,1000);
                
                plot(d_U_BS*cos(t),d_U_BS*sin(t),'k--'); % Cell #1
                plot(d_U_BS*cos(t)+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(t)+r_m*cos(30*pi/180),'k--'); % Cell #2
                if L==7
                    plot(d_U_BS*cos(t),d_U_BS*sin(t)+(sqrt(3)*r_m),'k--'); % Cell #3
                    plot(d_U_BS*cos(t)-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(t)+r_m*cos(30*pi/180),'k--'); % Cell #4
                    plot(d_U_BS*cos(t)-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(t)-r_m*cos(30*pi/180),'k--'); % Cell #5
                    plot(d_U_BS*cos(t),d_U_BS*sin(t)-(sqrt(3)*r_m),'k--'); % Cell #6
                    plot(d_U_BS*cos(t)+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(t)-r_m*cos(30*pi/180),'k--'); % Cell #7
                end
                
                %%% User location %%%
                for U_aux=1:U
                    plot(d_U_BS*cos(mean_AOA_ini(U_aux,1)),d_U_BS*sin(mean_AOA_ini(U_aux,1)),'ko'); % Cell #1
                    plot(d_U_BS*cos(mean_AOA_ini(U_aux,2))+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini(U_aux,2))+r_m*cos(30*pi/180),'ko'); % Cell #2
                    if L==7
                        plot(d_U_BS*cos(mean_AOA_ini(U_aux,3)),d_U_BS*sin(mean_AOA_ini(U_aux,3))+(sqrt(3)*r_m),'ko'); % Cell #3
                        plot(d_U_BS*cos(mean_AOA_ini(U_aux,4))-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini(U_aux,4))+r_m*cos(30*pi/180),'ko'); % Cell #4
                        plot(d_U_BS*cos(mean_AOA_ini(U_aux,5))-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini(U_aux,5))-r_m*cos(30*pi/180),'ko'); % Cell #5
                        plot(d_U_BS*cos(mean_AOA_ini(U_aux,6)),d_U_BS*sin(mean_AOA_ini(U_aux,6))-(sqrt(3)*r_m),'ko'); % Cell #6
                        plot(d_U_BS*cos(mean_AOA_ini(U_aux,7))+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini(U_aux,7))-r_m*cos(30*pi/180),'ko'); % Cell #7
                    end
                end
                
                %%% Base Stations %%%
                plot(0,0,'k^'); % Cell #1
                plot((r_m+r_m*sin(30*pi/180)),r_m*cos(30*pi/180),'k^'); % Cell #2
                if L==7
                    plot(0,(sqrt(3)*r_m),'k^'); % Cell #3
                    plot(-(r_m+r_m*sin(30*pi/180)),r_m*cos(30*pi/180),'k^'); % Cell #4
                    plot(-(r_m+r_m*sin(30*pi/180)),-r_m*cos(30*pi/180),'k^'); % Cell #5
                    plot(0,-(sqrt(3)*r_m),'k^'); % Cell #6
                    plot((r_m+r_m*sin(30*pi/180)),-r_m*cos(30*pi/180),'k^'); % Cell #7
                end
                
                axis([-3*r_m 3*r_m -3*r_m 3*r_m]);
                
                xlabel({'Geographical distance [km]'},'Interpreter','latex')
                ylabel({'Geographical distance [km]'},'Interpreter','latex')
                
            end
            
            aux_M=0;
            for M=M_ini % Number of antennas "for loop".
                
                if different_sigman2==1
                    sigman2=0.5 + (1.5-0.5).*rand(1,M);
                else
                    sigman2=sigman2_aux_ini*ones(1,M);
                end
                
                aux_M=aux_M+1;
                M_plot(aux_M)=M;
                
                sigman2_sum_crlb(aux_M)=(norm(sigman2,'fro')^2);
                
                if error_vs_ant_num==1 || rate_vs_ant_num==1
                    fprintf('Realization %d, M=%d\n',cell_edge_dist_aux,M)
                end
                
                R_cap=zeros(M,M,L,L,U);
                
                distances_gen=zeros(L,L,U);
                mean_AOA_gen=zeros(U,SL,L,L);
                for U_cap=1:U
                    
                    %% Distance from users to the desired Base Station (m), Pilot sequence assigned to the first user %%
                    if L==2
                        distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (0) )^2 )];
                        distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )];
                    end
                    
                    if L==7
                        distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (0) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (0) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (0) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (0) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (0) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (0) )^2 )];
                        
                        distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                        
                        distances(3,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - ((sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )];
                        
                        distances(4,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                        
                        distances(5,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )];
                        
                        distances(6,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-(sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )];
                        
                        distances(7,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )];
                    end
                    
                    %% AOAs, Pilot sequence assigned to the first user %%
                    if random_AOA==0
                        mean_AOA=zeros(U,SL,L,L);
                        if L==2
                            mean_AOA(U_cap,1,:,1)=[mean_AOA_ini(U_cap,1) atan2(( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ))];
                            mean_AOA(U_cap,1,:,2)=[mean_AOA_ini(U_cap,2) atan2(( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ))];
                        end
                        
                        if L==7
                            mean_AOA(U_cap,1,:,1)=[mean_AOA_ini(U_cap,1) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (0) )  )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                            
                            mean_AOA(U_cap,1,:,2)=[mean_AOA_ini(U_cap,2) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                            
                            mean_AOA(U_cap,1,:,3)=[mean_AOA_ini(U_cap,3) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                            
                            mean_AOA(U_cap,1,:,4)=[mean_AOA_ini(U_cap,4) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                            
                            mean_AOA(U_cap,1,:,5)=[mean_AOA_ini(U_cap,5) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                            
                            mean_AOA(U_cap,1,:,6)=[mean_AOA_ini(U_cap,6) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                            
                            mean_AOA(U_cap,1,:,7)=[mean_AOA_ini(U_cap,7) atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                atan2( ( (d_U_BS*sin(mean_AOA_ini(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                        end
                    end
                    
                    for aux_L=1:L
                        if L==2
                            if aux_L==1
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,2);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                end
                            else
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,1);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                end
                            end
                        else
                            if aux_L==1
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            elseif aux_L==2
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            elseif aux_L==3
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            elseif aux_L==4
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            elseif aux_L==5
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            elseif aux_L==6
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,1);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                end
                            else
                                distances_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                distances_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                distances_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                distances_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                distances_gen(aux_L,5,U_cap)=distances(aux_L,6);
                                distances_gen(aux_L,6,U_cap)=distances(aux_L,7);
                                distances_gen(aux_L,7,U_cap)=distances(aux_L,1);
                                for aux_SL=1:SL
                                    mean_AOA_gen(U_cap,aux_SL,1,aux_L)=mean_AOA(U_cap,aux_SL,2,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,2,aux_L)=mean_AOA(U_cap,aux_SL,3,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,3,aux_L)=mean_AOA(U_cap,aux_SL,4,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,4,aux_L)=mean_AOA(U_cap,aux_SL,5,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,5,aux_L)=mean_AOA(U_cap,aux_SL,6,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,6,aux_L)=mean_AOA(U_cap,aux_SL,7,aux_L);
                                    mean_AOA_gen(U_cap,aux_SL,7,aux_L)=mean_AOA(U_cap,aux_SL,1,aux_L);
                                end
                            end
                        end
                    end
                    
                    %% Estimation of channel covariance matrices
                    sum_R=zeros(M,M,L);
                    R=zeros(M,M*L,L); % [R11 R21 ... RL1 R12 R22 ... RL2].
                    if same_Ptx==0
                        if scenario == 1
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        elseif scenario == 2
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        elseif scenario == 3
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((34.53 + 38*log10(d_U_BS))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        else
                            alpha_delta=1*(10^((SNR_dB)/10))*(d_U_BS^gamma)*(mean(sigman2)/tao)*(M*tao)/M; % Constant dependent on the prescribed average SNR.
                        end
                    else
                        alpha_delta=Ptx;
                    end
                    for aux_L=1:L % Estimation of L covariance matrices for L cells.
                        
                        for R_estimation_aux=1:R_estimation_rep % Training repetitions.
                            
                            alpha=zeros(TC,SP,L); % Path attenuation.
                            for l=1:L
                                d=distances(aux_L,l);
                                if scenario == 1
                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                    beta=alpha_delta/(10^(PL/10));
                                elseif scenario == 2
                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                    beta=alpha_delta/(10^(PL/10));
                                elseif scenario == 3
                                    PL = 34.53 + 38*log10(d);
                                    beta=alpha_delta/(10^(PL/10));
                                else
                                    beta=alpha_delta/(d^gamma); % Distance-based path loss.
                                end
                                alpha(:,:,l)=(1/sqrt(2))*sqrt(beta)*randn(TC,SP)+1i*(1/sqrt(2))*sqrt(beta)*randn(TC,SP);
                            end
                            
                            theta=zeros(TC,SP,L); % Random AOA.
                            for l=1:L
                                for aux_TC=1:TC
                                    for aux_SP=1:SP
                                        if AS_active==1
                                            if strcmp(AOA_dist,'Gaussian')==1
                                                theta(aux_TC,aux_SP,l)=mean_AOA(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA*randn(1,1);
                                            else
                                                theta(aux_TC,aux_SP,l)=(mean_AOA(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA) + ((mean_AOA(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA)-(mean_AOA(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA)).*rand(1,1);
                                            end
                                        else
                                            theta(aux_TC,aux_SP,l)=mean_AOA(U_cap,aux_TC,l,aux_L);
                                        end
                                    end
                                end
                            end
                            
                            h=zeros(M,L); % L channel vectors.
                            for l=1:L
                                for aux_TC=1:TC
                                    for aux_SP=1:SP
                                        a=exp(-1i*2*pi*D/lambda.*((0:M-1).')*cos(theta(aux_TC,aux_SP,l))); % Steering vector.
                                        h(:,l)=h(:,l)+(a*alpha(aux_TC,aux_SP,l));
                                    end
                                end
                                h(:,l)=h(:,l)/sqrt(TC*SP);
                            end
                            
                            for l=1:L
                                R(:,M*(l-1)+1:l*M,aux_L)=R(:,M*(l-1)+1:l*M,aux_L)+h(:,l)*h(:,l)';
                            end
                            
                        end
                        R(:,:,aux_L)=R(:,:,aux_L)./R_estimation_rep;
                        
                        for l=1:L
                            sum_R(:,:,aux_L)=sum_R(:,:,aux_L)+R(:,M*(l-1)+1:l*M,aux_L);
                        end
                        
                        if L==2
                            
                            if aux_L==1
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                
                            else
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                
                            end
                            
                        else
                            
                            if aux_L==1
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            elseif aux_L==2
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            elseif aux_L==3
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            elseif aux_L==4
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            elseif aux_L==5
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            elseif aux_L==6
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                            else
                                R_cap(:,:,aux_L,1,U_cap)=R(:,M*(2-1)+1:2*M,aux_L);
                                R_cap(:,:,aux_L,2,U_cap)=R(:,M*(3-1)+1:3*M,aux_L);
                                R_cap(:,:,aux_L,3,U_cap)=R(:,M*(4-1)+1:4*M,aux_L);
                                R_cap(:,:,aux_L,4,U_cap)=R(:,M*(5-1)+1:5*M,aux_L);
                                R_cap(:,:,aux_L,5,U_cap)=R(:,M*(6-1)+1:6*M,aux_L);
                                R_cap(:,:,aux_L,6,U_cap)=R(:,M*(7-1)+1:7*M,aux_L);
                                R_cap(:,:,aux_L,7,U_cap)=R(:,M*(1-1)+1:1*M,aux_L);
                            end
                            
                        end
                        
                    end
                    
                    if comp_cap==0
                        break;
                    end
                    
                end
                
                R=R_cap;
                R_aux=R;
                
                %% Coordinated Pilot Assignment %%
                if comp_coor==1
                    mean_AOA_ini_aux=zeros(U,L);
                    mean_AOA_aux=zeros(U,SL,L,L);
                    user_taken=zeros(U,L);
                    for U_aux_cap=1:U
                        F=[];
                        user=ones(1,L);
                        for l=1:L
                            for u=1:U
                                user_taken_aux=1;
                                for u2=1:U
                                    if user_taken(u2,l)==u;
                                        user_taken_aux=0;
                                        break;
                                    end
                                end
                                if user_taken_aux==1
                                    user(l)=u;
                                    break;
                                end
                            end
                        end
                        for L_aux=1:L
                            
                            for U_aux=1:U
                                
                                user(L_aux)=U_aux;
                                
                                user_taken_aux=1;
                                for u=1:U
                                    if user_taken(u,L_aux)==user(L_aux);
                                        user_taken_aux=0;
                                    end
                                end
                                
                                if user_taken_aux==1
                                    
                                    %% Distance from users to the desired Base Station (m) %%
                                    if L==2
                                        distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) )^2 )];
                                        distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) )^2 )];
                                    end
                                    
                                    if L==7
                                        distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (0) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (0) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (0) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (0) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (0) )^2 )];
                                        
                                        distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                                        
                                        distances(3,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - ((sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )];
                                        
                                        distances(4,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                                        
                                        distances(5,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )];
                                        
                                        distances(6,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-(sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )];
                                        
                                        distances(7,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                            sqrt( ( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )];
                                    end
                                    
                                    %% AOAs %%
                                    if random_AOA==0
                                        mean_AOA=zeros(1,SL,L,L);
                                        if L==2
                                            mean_AOA(1,1,:,1)=[mean_AOA_ini(user(1),1) atan2(( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ))];
                                            mean_AOA(1,1,:,2)=[mean_AOA_ini(user(2),2) atan2(( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ))];
                                        end
                                        
                                        if L==7
                                            mean_AOA(1,1,:,1)=[mean_AOA_ini(user(1),1) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) )  )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                            
                                            mean_AOA(1,1,:,2)=[mean_AOA_ini(user(2),2) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                                            
                                            mean_AOA(1,1,:,3)=[mean_AOA_ini(user(3),3) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                            
                                            mean_AOA(1,1,:,4)=[mean_AOA_ini(user(4),4) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                            
                                            mean_AOA(1,1,:,5)=[mean_AOA_ini(user(5),5) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                            
                                            mean_AOA(1,1,:,6)=[mean_AOA_ini(user(6),6) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                            
                                            mean_AOA(1,1,:,7)=[mean_AOA_ini(user(7),7) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                                atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                                        end
                                    end
                                    
                                    F_coor=0;
                                    
                                    %% Estimation of channel covariance matrices
                                    sum_R_coor=zeros(M,M,L);
                                    R_coor=zeros(M,M*L,L);
                                    if same_Ptx==0
                                        if scenario == 1
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        elseif scenario == 2
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        elseif scenario == 3
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((34.53 + 38*log10(d_U_BS))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        else
                                            alpha_delta=1*(10^((SNR_dB)/10))*(d_U_BS^gamma)*(mean(sigman2)/tao)*(M*tao)/M; % Constant dependent on the prescribed average SNR.
                                        end
                                    else
                                        alpha_delta=Ptx;
                                    end
                                    
                                    for aux_L=1:L_aux
                                        
                                        for R_estimation_aux=1:R_estimation_rep % Training repetitions.
                                            
                                            alpha=zeros(TC,SP,L); % Path attenuation.
                                            for l=1:L
                                                d=distances(aux_L,l);
                                                if scenario == 1
                                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                                    beta=alpha_delta/(10^(PL/10));
                                                elseif scenario == 2
                                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                                    beta=alpha_delta/(10^(PL/10));
                                                elseif scenario == 3
                                                    PL = 34.53 + 38*log10(d);
                                                    beta=alpha_delta/(10^(PL/10));
                                                else
                                                    beta=alpha_delta/(d^gamma); % Distance-based path loss.
                                                end
                                                alpha(:,:,l)=(1/sqrt(2))*sqrt(beta)*randn(TC,SP)+1i*(1/sqrt(2))*sqrt(beta)*randn(TC,SP);
                                            end
                                            
                                            theta=zeros(TC,SP,L); % Random AOA.
                                            for l=1:L
                                                for aux_TC=1:TC
                                                    for aux_SP=1:SP
                                                        if random_AOA==1
                                                            if AS_active==1
                                                                if strcmp(AOA_dist,'Gaussian')==1
                                                                    theta(aux_TC,aux_SP,l)=(mean_AOA(user(l),round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA*randn(1,1));
                                                                else
                                                                    theta(aux_TC,aux_SP,l)= (mean_AOA(user(l),round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA) + ((mean_AOA(user(l),round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA)-(mean_AOA(user(l),round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA)).*rand(1,1);
                                                                end
                                                            else
                                                                theta(aux_TC,aux_SP,l)=mean_AOA(user(l),aux_TC,l,aux_L);
                                                            end
                                                        else
                                                            if AS_active==1
                                                                if strcmp(AOA_dist,'Gaussian')==1
                                                                    theta(aux_TC,aux_SP,l)=(mean_AOA(1,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA*randn(1,1));
                                                                else
                                                                    theta(aux_TC,aux_SP,l)= (mean_AOA(1,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA) + ((mean_AOA(1,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA)-(mean_AOA(1,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA)).*rand(1,1);
                                                                end
                                                            else
                                                                theta(aux_TC,aux_SP,l)=mean_AOA(1,aux_TC,l,aux_L);
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                            
                                            h=zeros(M,L);
                                            for l=1:L
                                                for aux_TC=1:TC
                                                    for aux_SP=1:SP
                                                        a=exp(-1i*2*pi*D/lambda.*((0:M-1).')*cos(theta(aux_TC,aux_SP,l))); % Steering vector.
                                                        h(:,l)=h(:,l)+(a*alpha(aux_TC,aux_SP,l));
                                                    end
                                                end
                                                h(:,l)=h(:,l)/sqrt(TC*SP);
                                            end
                                            
                                            for l=1:L
                                                R_coor(:,M*(l-1)+1:l*M,aux_L)=R_coor(:,M*(l-1)+1:l*M,aux_L)+h(:,l)*h(:,l)';
                                            end
                                            
                                        end
                                        R_coor(:,:,aux_L)=R_coor(:,:,aux_L)./R_estimation_rep;
                                        
                                        for l=1:L
                                            sum_R_coor(:,:,aux_L)=sum_R_coor(:,:,aux_L)+R_coor(:,M*(l-1)+1:l*M,aux_L);
                                        end
                                        
                                        MSE=trace( R_coor(:,1:M,aux_L) - (R_coor(:,1:M,aux_L)^2) / ( (diag(sigman2)/tao) + sum_R_coor(:,:,aux_L) ) ) / trace (R_coor(:,1:M,aux_L));
                                        
                                        F_coor=F_coor+MSE;
                                        
                                    end
                                    
                                else
                                    F_coor=Inf;
                                end
                                
                                F=[F F_coor]; %#ok<AGROW>
                                
                            end
                            [min_F, min_F_ind]=min(F);
                            user(L_aux)=min_F_ind;
                            F=[];
                            
                        end
                        
                        for l=1:L
                            mean_AOA_ini_aux(U_aux_cap,l)=mean_AOA_ini(user(l),l);
                            user_taken(U_aux_cap,l)=user(l);
                        end
                        
                        if random_AOA==0
                            mean_AOA=zeros(1,SL,L,L);
                            if L==2
                                mean_AOA(1,1,:,1)=[mean_AOA_ini(user(1),1) atan2(( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ))];
                                mean_AOA(1,1,:,2)=[mean_AOA_ini(user(2),2) atan2(( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ))];
                            end
                            if L==7
                                mean_AOA(1,1,:,1)=[mean_AOA_ini(user(1),1) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) )  )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA(1,1,:,2)=[mean_AOA_ini(user(2),2) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA(1,1,:,3)=[mean_AOA_ini(user(3),3) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA(1,1,:,4)=[mean_AOA_ini(user(4),4) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA(1,1,:,5)=[mean_AOA_ini(user(5),5) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA(1,1,:,6)=[mean_AOA_ini(user(6),6) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(7),7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini(user(7),7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA(1,1,:,7)=[mean_AOA_ini(user(7),7) atan2( ( (d_U_BS*sin(mean_AOA_ini(user(1),1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(1),1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(2),2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(2),2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(3),3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(3),3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(4),4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(4),4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(5),5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(5),5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini(user(6),6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini(user(6),6))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                            end
                            for aux_L=1:L
                                for l=1:L
                                    mean_AOA_aux(U_aux_cap,:,l,aux_L)=mean_AOA(1,:,l,aux_L);
                                end
                            end
                        else
                            for aux_L=1:L
                                for l=1:L
                                    mean_AOA_aux(U_aux_cap,:,l,aux_L)=mean_AOA(user(l),:,l,aux_L);
                                end
                            end
                        end
                        
                        if comp_cap==0
                            break;
                        end
                        
                    end
                    
                    if plot_network==1
                        %%% User location %%%
                        for U_aux=1:U
                            switch U_aux
                                case 1,
                                    s='bs';
                                case 2,
                                    s='gs';
                                case 3,
                                    s='rs';
                                case 4,
                                    s='cs';
                                case 5,
                                    s='ms';
                                case 6,
                                    s='ys';
                                case 7,
                                    s='ks';
                                case 8,
                                    s='bo';
                                case 9,
                                    s='go';
                                case 10,
                                    s='ro';
                            end
                            plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,1)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,1)),s); % Cell #1
                            plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,2))+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,2))+r_m*cos(30*pi/180),s); % Cell #2
                            if L==7
                                plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,3)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,3))+(sqrt(3)*r_m),s); % Cell #3
                                plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,4))-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,4))+r_m*cos(30*pi/180),s); % Cell #4
                                plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,5))-(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,5))-r_m*cos(30*pi/180),s); % Cell #5
                                plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,6)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,6))-(sqrt(3)*r_m),s); % Cell #6
                                plot(d_U_BS*cos(mean_AOA_ini_aux(U_aux,7))+(r_m+r_m*sin(30*pi/180)),d_U_BS*sin(mean_AOA_ini_aux(U_aux,7))-r_m*cos(30*pi/180),s); % Cell #7
                            end
                        end
                    end
                    
                    R_cap=zeros(M,M,L,L,U);
                    
                    distances_coor_gen=zeros(L,L,U);
                    mean_AOA_coor_gen=zeros(U,SL,L,L);
                    for U_cap=1:U
                        
                        %% Distance from users to the desired Base Station (m), Pilot sequence assigned to the first user %%
                        if L==2
                            distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (0) )^2 )];
                            distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )];
                        end
                        
                        if L==7
                            distances(1,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (0) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (0) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (0) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (0) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (0) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (0) )^2 )];
                            
                            distances(2,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                            
                            distances(3,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - ((sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) )^2 )];
                            
                            distances(4,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) )^2 )];
                            
                            distances(5,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )];
                            
                            distances(6,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-(sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) )^2 )];
                            
                            distances(7,:)=[d_U_BS sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) )^2 )...
                                sqrt( ( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) )^2 + ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) )^2 )];
                        end
                        
                        %% AOAs, Coordinated Pilot Assignment %%
                        if random_AOA==0
                            mean_AOA_aux=zeros(U,SL,L,L);
                            if L==2
                                mean_AOA_aux(U_cap,1,:,1)=[mean_AOA_ini_aux(U_cap,1) atan2(( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ))];
                                mean_AOA_aux(U_cap,1,:,2)=[mean_AOA_ini_aux(U_cap,2) atan2(( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ))];
                            end
                            
                            if L==7
                                mean_AOA_aux(U_cap,1,:,1)=[mean_AOA_ini_aux(U_cap,1) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (0) )  )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (0) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,2)=[mean_AOA_ini_aux(U_cap,2) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,3)=[mean_AOA_ini_aux(U_cap,3) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - ((sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,4)=[mean_AOA_ini_aux(U_cap,4) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,5)=[mean_AOA_ini_aux(U_cap,5) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - (-(r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (-(r_m+r_m*sin(30*pi/180))) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,6)=[mean_AOA_ini_aux(U_cap,6) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - (0) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,7))-r_m*cos(30*pi/180)) - (-(sqrt(3)*r_m)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,7))+(r_m+r_m*sin(30*pi/180))) - (0) ) )];
                                
                                mean_AOA_aux(U_cap,1,:,7)=[mean_AOA_ini_aux(U_cap,7) atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,1))) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,1))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,2))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,2))+(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,3))+(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,3))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,4))+r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,4))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,5))-r_m*cos(30*pi/180)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,5))-(r_m+r_m*sin(30*pi/180))) - ((r_m+r_m*sin(30*pi/180))) ) )...
                                    atan2( ( (d_U_BS*sin(mean_AOA_ini_aux(U_cap,6))-(sqrt(3)*r_m)) - (-r_m*cos(30*pi/180)) ),( (d_U_BS*cos(mean_AOA_ini_aux(U_cap,6))) - ((r_m+r_m*sin(30*pi/180))) ) )];
                            end
                        end
                        
                        for aux_L=1:L
                            if L==2
                                if aux_L==1
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,2);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                    end
                                else
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,1);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                    end
                                end
                            else
                                if aux_L==1
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                elseif aux_L==2
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                elseif aux_L==3
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                elseif aux_L==4
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                elseif aux_L==5
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                elseif aux_L==6
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,1);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,7);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                    end
                                else
                                    distances_coor_gen(aux_L,1,U_cap)=distances(aux_L,2);
                                    distances_coor_gen(aux_L,2,U_cap)=distances(aux_L,3);
                                    distances_coor_gen(aux_L,3,U_cap)=distances(aux_L,4);
                                    distances_coor_gen(aux_L,4,U_cap)=distances(aux_L,5);
                                    distances_coor_gen(aux_L,5,U_cap)=distances(aux_L,6);
                                    distances_coor_gen(aux_L,6,U_cap)=distances(aux_L,7);
                                    distances_coor_gen(aux_L,7,U_cap)=distances(aux_L,1);
                                    for aux_SL=1:SL
                                        mean_AOA_coor_gen(U_cap,aux_SL,1,aux_L)=mean_AOA_aux(U_cap,aux_SL,2,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,2,aux_L)=mean_AOA_aux(U_cap,aux_SL,3,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,3,aux_L)=mean_AOA_aux(U_cap,aux_SL,4,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,4,aux_L)=mean_AOA_aux(U_cap,aux_SL,5,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,5,aux_L)=mean_AOA_aux(U_cap,aux_SL,6,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,6,aux_L)=mean_AOA_aux(U_cap,aux_SL,7,aux_L);
                                        mean_AOA_coor_gen(U_cap,aux_SL,7,aux_L)=mean_AOA_aux(U_cap,aux_SL,1,aux_L);
                                    end
                                end
                            end
                        end
                        
                        %% Estimation of channel covariance matrices (Coordinated Pilot Assignment)
                        sum_R_coor=zeros(M,M,L);
                        R_coor=zeros(M,M*L,L);
                        if same_Ptx==0
                            if scenario == 1
                                alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                            elseif scenario == 2
                                alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                            elseif scenario == 3
                                alpha_delta=1*(10^((SNR_dB)/10))*((10^((34.53 + 38*log10(d_U_BS))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                            else
                                alpha_delta=1*(10^((SNR_dB)/10))*(d_U_BS^gamma)*(mean(sigman2)/tao)*(M*tao)/M; % Constant dependent on the prescribed average SNR.
                            end
                        else
                            alpha_delta=Ptx;
                        end
                        
                        for aux_L=1:L
                            
                            for R_estimation_aux=1:R_estimation_rep % Training repetitions.
                                
                                alpha=zeros(TC,SP,L); % Path attenuation.
                                for l=1:L
                                    d=distances(aux_L,l);
                                    if scenario == 1
                                        PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                        beta=alpha_delta/(10^(PL/10));
                                    elseif scenario == 2
                                        PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                        beta=alpha_delta/(10^(PL/10));
                                    elseif scenario == 3
                                        PL = 34.53 + 38*log10(d);
                                        beta=alpha_delta/(10^(PL/10));
                                    else
                                        beta=alpha_delta/(d^gamma); % Distance-based path loss.
                                    end
                                    alpha(:,:,l)=(1/sqrt(2))*sqrt(beta)*randn(TC,SP)+1i*(1/sqrt(2))*sqrt(beta)*randn(TC,SP);
                                end
                                
                                theta=zeros(TC,SP,L); % Random AOA.
                                for l=1:L
                                    for aux_TC=1:TC
                                        for aux_SP=1:SP
                                            if AS_active==1
                                                if strcmp(AOA_dist,'Gaussian')==1
                                                    theta(aux_TC,aux_SP,l)=(mean_AOA_aux(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA*randn(1,1));
                                                else
                                                    theta(aux_TC,aux_SP,l)=(mean_AOA_aux(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA) + ((mean_AOA_aux(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA)-(mean_AOA_aux(U_cap,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA)).*rand(1,1);
                                                end
                                            else
                                                theta(aux_TC,aux_SP,l)=mean_AOA_aux(U_cap,aux_TC,l,aux_L);
                                            end
                                        end
                                    end
                                end
                                
                                h=zeros(M,L);
                                for l=1:L
                                    for aux_TC=1:TC
                                        for aux_SP=1:SP
                                            a=exp(-1i*2*pi*D/lambda.*((0:M-1).')*cos(theta(aux_TC,aux_SP,l))); % Steering vector.
                                            h(:,l)=h(:,l)+(a*alpha(aux_TC,aux_SP,l));
                                        end
                                    end
                                    h(:,l)=h(:,l)/sqrt(TC*SP);
                                end
                                
                                for l=1:L
                                    R_coor(:,M*(l-1)+1:l*M,aux_L)=R_coor(:,M*(l-1)+1:l*M,aux_L)+h(:,l)*h(:,l)';
                                end
                                
                            end
                            R_coor(:,:,aux_L)=R_coor(:,:,aux_L)./R_estimation_rep;
                            
                            for l=1:L
                                sum_R_coor(:,:,aux_L)=sum_R_coor(:,:,aux_L)+R_coor(:,M*(l-1)+1:l*M,aux_L);
                            end
                            
                            if L==2
                                
                                if aux_L==1
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    
                                else
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    
                                end
                                
                            else
                                
                                if aux_L==1
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                elseif aux_L==2
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                elseif aux_L==3
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                elseif aux_L==4
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                elseif aux_L==5
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                elseif aux_L==6
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                else
                                    R_cap(:,:,aux_L,1,U_cap)=R_coor(:,M*(2-1)+1:2*M,aux_L);
                                    R_cap(:,:,aux_L,2,U_cap)=R_coor(:,M*(3-1)+1:3*M,aux_L);
                                    R_cap(:,:,aux_L,3,U_cap)=R_coor(:,M*(4-1)+1:4*M,aux_L);
                                    R_cap(:,:,aux_L,4,U_cap)=R_coor(:,M*(5-1)+1:5*M,aux_L);
                                    R_cap(:,:,aux_L,5,U_cap)=R_coor(:,M*(6-1)+1:6*M,aux_L);
                                    R_cap(:,:,aux_L,6,U_cap)=R_coor(:,M*(7-1)+1:7*M,aux_L);
                                    R_cap(:,:,aux_L,7,U_cap)=R_coor(:,M*(1-1)+1:1*M,aux_L);
                                end
                                
                            end
                            
                        end
                        
                        if comp_cap==0
                            break;
                        end
                        
                    end
                end
                
                R=R_cap;
                
                if R_ideal_aux==1
                    R_ideal_1=zeros(M,M,L,L,U);
                    R_ideal_2=zeros(M,M,L,L,U);
                    delta_theta=sigma_theta_AOA;
                    delta=D/lambda;
                    if same_Ptx==0
                        if scenario == 1
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        elseif scenario == 2
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        elseif scenario == 3
                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((34.53 + 38*log10(d_U_BS))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                        else
                            alpha_delta=1*(10^((SNR_dB)/10))*(d_U_BS^gamma)*(mean(sigman2)/tao)*(M*tao)/M; % Constant dependent on the prescribed average SNR.
                        end
                    else
                        alpha_delta=Ptx;
                    end
                    for j=1:L
                        for l=1:L
                            for k=1:U
                                for aux_SL=1:SL
                                    for m=1:M
                                        for n=1:M
                                            if strcmp(AOA_dist,'Gaussian')==1
                                                sum1=besselj(0,2*pi*delta*(m-n));
                                                sum2=0;
                                                sum3=0;
                                                for sum_aux=1:R_bessel_series
                                                    sum2=sum2+besselj(2*sum_aux,2*pi*delta*(m-n))*exp(-2*(sum_aux^2)*sigma_theta_AOA^2)*cos(2*sum_aux*mean_AOA_gen(k,aux_SL,l,j));
                                                    sum3=sum3+besselj(2*sum_aux-1,2*pi*delta*(m-n))*exp(-0.5*((2*sum_aux-1)^2)*sigma_theta_AOA^2)*sin((2*sum_aux-1)*mean_AOA_gen(k,aux_SL,l,j));
                                                end
                                                R_ideal_1(m,n,j,l,k)=R_ideal_1(m,n,j,l,k)+sum1+2*sum2+2*1i*sum3;
                                                R_ideal_2(m,n,j,l,k)=R_ideal_2(m,n,j,l,k)+0;
                                            else
                                                sum1=besselj(0,2*pi*delta*(m-n));
                                                sum2=0;
                                                sum3=0;
                                                for sum_aux=1:R_bessel_series
                                                    sum2=sum2+besselj(2*sum_aux,2*pi*delta*(m-n))*((cos(2*sum_aux*mean_AOA_gen(k,aux_SL,l,j))*sin(2*sum_aux*delta_theta))/(sum_aux));
                                                    sum3=sum3+besselj(2*sum_aux-1,2*pi*delta*(m-n))*((sin((2*sum_aux-1)*mean_AOA_gen(k,aux_SL,l,j))*sin((2*sum_aux-1)*delta_theta))/(2*sum_aux-1));
                                                end
                                                R_ideal_1(m,n,j,l,k)=R_ideal_1(m,n,j,l,k)+(sum1+(1/delta_theta)*sum2+((2*1i)/delta_theta)*sum3);
                                                
                                                R_ideal_2(m,n,j,l,k)=R_ideal_2(m,n,j,l,k)+(exp(1i*2*pi*(m-n)*delta*sin(mean_AOA_gen(k,aux_SL,l,j)))*sinc(2*(m-n)*delta*delta_theta*cos(mean_AOA_gen(k,aux_SL,l,j))));
                                            end
                                        end
                                    end
                                end
                                d=distances_gen(j,l,k);
                                if scenario == 1
                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                    beta_i=alpha_delta/(10^(PL/10));
                                elseif scenario == 2
                                    PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                    beta_i=alpha_delta/(10^(PL/10));
                                elseif scenario == 3
                                    PL = 34.53 + 38*log10(d);
                                    beta_i=alpha_delta/(10^(PL/10));
                                else
                                    beta_i=alpha_delta/(d^gamma); % Distance-based path loss.
                                end
                                R_ideal_1(:,:,j,l,k)=R_ideal_1(:,:,j,l,k)*beta_i;
                                R_ideal_2(:,:,j,l,k)=R_ideal_2(:,:,j,l,k)*beta_i;
                            end
                        end
                    end
                    R_ideal_1=R_ideal_1/SL;
                    R_ideal_2=R_ideal_2/SL;
                    
                    err_R_num_1=((norm(R_ideal_1(:,:,1,1,1)-R(:,:,1,1,1),'fro')^2));
                    err_R_den_1=((norm(R(:,:,1,1,1),'fro')^2));
                    err_R_num_2=((norm(R_ideal_2(:,:,1,1,1)-R(:,:,1,1,1),'fro')^2));
                    err_R_den_2=((norm(R(:,:,1,1,1),'fro')^2));
                    R=R_ideal_1;
                    R_aux=R_ideal_1;
                end
                
                R_aux_coor=R;
                
                R=R_aux;
                
                sum_R=zeros(M,M,L,U);
                for j=1:L
                    for k=1:U
                        for l=1:L
                            sum_R(:,:,j,k)=sum_R(:,:,j,k)+R(:,:,j,l,k);
                        end
                    end
                end
                
                R_coor=R_aux_coor;
                sum_R_coor=zeros(M,M,L,U);
                for j=1:L
                    for k=1:U
                        for l=1:L
                            sum_R_coor(:,:,j,k)=sum_R_coor(:,:,j,k)+R_coor(:,:,j,l,k);
                        end
                    end
                end
                
                if comp_rank==1
                    %% Rank of the channel covariance matrix
                    rank_R(1,aux_M)=rank_R(1,aux_M)+rank(R_coor(:,:,1,1,1));
                    rank_R(2,aux_M)=rank_R(2,aux_M)+cond(R_coor(:,:,1,1,1));
                    
                    mse_R(aux_M)=trace(R_coor(:,:,1,1,1)-(R_coor(:,:,1,1,1)^2)*((diag(sigman2)/tao)+sum_R_coor(:,:,1,1))^(-1));
                end
                
                %% Capacity: Theoretical computation
                if comp_cap==1 && comp_cap_theo==1
                    
                    J=1;
                    
                    Q=zeros(M,M,L,U);
                    for j=1:L
                        for k=1:U
                            Q(:,:,j,k)=((diag(sigman2)+tao*sum_R_coor(:,:,j,k))^(-1));
                        end
                    end
                    
                    PHI=zeros(M,M,L,L,U);
                    for j=1:L
                        for l=1:L
                            for k=1:U
                                PHI(:,:,j,l,k)=R_coor(:,:,j,j,k)*Q(:,:,j,k)*R_coor(:,:,j,l,k);
                            end
                        end
                    end
                    
                    lambda_b=zeros(1,L);
                    for l=1:L
                        sum_lambda_b=0;
                        for k=1:U
                            sum_lambda_b=sum_lambda_b+(1/M)*trace(PHI(:,:,l,l,k));
                        end
                        lambda_b(l)=((1/U)*sum_lambda_b)^(-1);
                    end
                    
                    for k2=1:U
                        
                        sum_BF1=0;
                        for l=1:L
                            for k=1:U
                                sum_BF1=sum_BF1+lambda_b(l)*(1/M)*trace(R_coor(:,:,l,J,k2)*PHI(:,:,l,l,k));
                            end
                        end
                        
                        sum_BF2=0;
                        for l=1:L
                            if l~=J
                                sum_BF2=sum_BF2+lambda_b(l)*abs((1/M)*trace(PHI(:,:,l,J,k2)))^2;
                            end
                        end
                        
                        R_BF_theoretical_coor_user(k2,aux_M)=R_BF_theoretical_coor_user(k2,aux_M)+log2(1+((lambda_b(J)*((1/M)*trace(PHI(:,:,J,J,k2)))^2) / ( (1/(rho*M)) + (1/(M)*sum_BF1) + (sum_BF2) )));
                        
                    end
                    
                    S=Z_dl/M;
                    rho_T=phi_dl;
                    delta_T_M=zeros(U,L);
                    for k=1:U
                        for l=1:L
                            delta_T_M(k,l)=limit_delta(M,PHI,U,S,rho_T,k,l,limit_delta_diff);
                        end
                    end
                    
                    T=zeros(M,M,L);
                    for l=1:L
                        sum_T=0;
                        for k=1:U
                            R_T=PHI(:,:,l,l,k);
                            delta_T=delta_T_M(k,l);
                            sum_T=sum_T+(R_T)/(1+delta_T);
                        end
                        T(:,:,l)=((1/M)*sum_T+S+rho_T*eye(M,M))^(-1);
                    end
                    
                    delta_T_p_M=zeros(U,U,L);
                    for k=1:U
                        for l=1:L
                            PHI_MAYUS=PHI(:,:,l,l,k);
                            delta_T_p_M(:,k,l)=delta_p(M,U,S,rho_T,PHI,T,l,PHI_MAYUS,delta_T_M);
                        end
                    end
                    
                    delta_T_b_p_M=zeros(U,U,L);
                    for k=1:U
                        for l=1:L
                            PHI_MAYUS=eye(M,M);
                            delta_T_b_p_M(:,k,l)=delta_p(M,U,S,rho_T,PHI,T,l,PHI_MAYUS,delta_T_M);
                        end
                    end
                    
                    g=zeros(L,L,U);
                    for l=1:L
                        for j=1:L
                            for k=1:U
                                g(l,j,k)=(1/M)*trace(PHI(:,:,l,j,k)*T(:,:,l));
                            end
                        end
                    end
                    
                    T_p=zeros(M,M,L,U);
                    for l=1:L
                        for k=1:U
                            PHI_MAYUS=PHI(:,:,l,l,k);
                            sum_T_p=0;
                            for k2=1:U
                                R_T_p=PHI(:,:,l,l,k2);
                                delta_T=delta_T_M(k2,l);
                                delta_T_p=delta_T_p_M(:,k,l);
                                sum_T_p=sum_T_p+(R_T_p*delta_T_p(k2)*T(:,:,l))/((1+delta_T)^2);
                            end
                            T_p(:,:,l,k)=T(:,:,l)*PHI_MAYUS*T(:,:,l)+T(:,:,l)*(1/M)*sum_T_p;
                        end
                        
                    end
                    
                    g_p=zeros(L,L,U,U);
                    for l=1:L
                        for j=1:L
                            for m=1:U
                                for k=1:U
                                    g_p(l,j,m,k)=(1/M)*trace(PHI(:,:,l,j,m)*T_p(:,:,l,k));
                                end
                            end
                        end
                    end
                    
                    T_b_p=zeros(M,M,L,U);
                    for l=1:L
                        for k=1:U
                            PHI_MAYUS=eye(M,M);
                            sum_T_p=0;
                            for k2=1:U
                                R_T_p=PHI(:,:,l,l,k2);
                                delta_T=delta_T_M(k2,l);
                                delta_T_p=delta_T_b_p_M(:,k,l);
                                sum_T_p=sum_T_p+(R_T_p*delta_T_p(k2)*T(:,:,l))/((1+delta_T)^2);
                            end
                            T_b_p(:,:,l,k)=T(:,:,l)*PHI_MAYUS*T(:,:,l)+T(:,:,l)*(1/M)*sum_T_p;
                        end
                    end
                    
                    lambda_b=zeros(1,L);
                    for l=1:L
                        lambda_b(l)=(U/M)*((1/M)*trace(T(:,:,l))-(1/M)*trace(((Z_dl/M)+phi_dl*eye(M,M))*T_b_p(:,:,l)))^(-1);
                    end
                    
                    mu=zeros(L,L,U,U);
                    for l=1:L
                        for j=1:L
                            for m=1:U
                                delta_T=delta_T_M(m,l);
                                for k=1:U
                                    delta_T_p=delta_T_p_M(:,k,l);
                                    mu(l,j,m,k)=(1/M)*trace(R_coor(:,:,l,j,m)*T_p(:,:,l,k))-(2*real(conj(g(l,j,m))*g_p(l,j,m,k))*(1+delta_T)-(abs(g(l,j,m))^2*delta_T_p(m)))/((1+delta_T)^2);
                                end
                            end
                        end
                    end
                    
                    for k2=1:U
                        
                        sum1=0;
                        sum_I_SINR_1=0;
                        for l=1:L
                            for k=1:U
                                sum1=sum1+lambda_b(l)*(((1+delta_T_M(k2,J))/(1+delta_T_M(k,l)))^2)*mu(l,J,k2,k);
                                sum_I_SINR_1=sum_I_SINR_1+lambda_b(l)*(((1)/(1+delta_T_M(k,l)))^2)*mu(l,J,k2,k);
                            end
                        end
                        
                        sum2=0;
                        sum_I_SINR_2=0;
                        for l=1:L
                            if l~=J
                                sum2=sum2+lambda_b(l)*(((1+delta_T_M(k2,J))/(1+delta_T_M(k2,l)))^2)*abs(g(l,J,k2))^2;
                                sum_I_SINR_2=sum_I_SINR_2+lambda_b(l)*M*(((1)/(1+delta_T_M(k2,l)))^2)*abs(g(l,J,k2))^2;
                            end
                        end
                        
                        R_RZF_theoretical_coor_user(k2,aux_M)=R_RZF_theoretical_coor_user(k2,aux_M)+log2(1+((lambda_b(J)*(delta_T_M(k2,J)^2))/( (((1+delta_T_M(k2,J))^2)/(rho*M)) + (1/M)*sum1 + sum2)));
                        N_SINR(k2,aux_M)=(1/rho);
                        S_SINR(k2,aux_M)=lambda_b(J)*M*(delta_T_M(k2,J)^2)/((1+delta_T_M(k2,J))^2);
                        Sp_SINR(k2,aux_M)=0;
                        I_SINR(k2,aux_M)=sum_I_SINR_1+sum_I_SINR_2;
                    end
                    
                end
                
                if comp_sim==1
                    
                    %% Initialization of estimators
                    h1_est_LS=zeros(M,1); % Least Squares (LS) estimator for h1 (desired channel) (L desired channels).
                    h1_est_LS_no_int=zeros(M,1); % Least Squares (LS) estimator for h1 (desired channel) without interference (same pilot sequence) (L desired channels).
                    h1_est_CB=zeros(M,1); % Covariance-aided Bayesian (CB) estimator for h1 (desired channel) (same pilot sequence) (L desired channels).
                    h1_est_CB_no_int=zeros(M,1); % Covariance-aided Bayesian (CB) estimator for h1 (desired channel) without interference (same pilot sequence) (L desired channels).
                    h1_est_CPA=zeros(M,1); % Coordinated Pilot Assignment (CPA) estimator for h1 (desired channel) (same pilot sequence) (L desired channels).
                    h1_est_CPA_noise_ML=zeros(M,1); % Coordinated Pilot Assignment (CPA) estimator for h1 (desired channel) (same pilot sequence) (L desired channels) (with noise variance estimation (ML) ).
                    h1_est_CPA_noise_MM=zeros(M,1); % Coordinated Pilot Assignment (CPA) estimator for h1 (desired channel) (same pilot sequence) (L desired channels) (with noise variance estimation (MM)).
                    h1_est_CPA_noise_NU_1dB=zeros(M,1);
                    
                    %%% MONTE CARLO SIMULATIONS %%%
                    for MC_aux=1:MC_rep
                        fprintf('%d\n',MC_aux)
                        
                        %% Pilot Sequences
                        if pilot_mod==1
                            cons_sym_aux=randi([0 M_cons-1],tao,L);
                            cons_sym=step(modulator,reshape(cons_sym_aux,tao*L,1));
                            cons_sym=reshape(cons_sym,tao,L);
                        else
                            orth_vectors=sqrt(tao)*orth(randn(tao)+1i*randn(tao)); % Orthogonal vectors (tao by tao).
                        end
                        
                        %% Pilot Sequences (unknown)
                        if pilot_mod==1
                            cons_sym_aux_mix=randi([0 M_cons-1],kappa,L);
                            cons_sym_mix=step(modulator,reshape(cons_sym_aux_mix,kappa*L,1));
                            cons_sym_mix=reshape(cons_sym_mix,kappa,L);
                        else
                            orth_vectors_mix=sqrt(kappa)*orth(randn(kappa)+1i*randn(kappa)); % Orthogonal vectors (tao by tao).
                        end
                        
                        % Pilot Sequences
                        s=zeros(tao,L); % L pilot sequences.
                        for s_aux=1:L
                            if pilot_mod==0
                                s(:,s_aux)=orth_vectors(:,1); % Same pilot sequences.
                            else
                                s(:,s_aux)=cons_sym(:,1);
                            end
                        end
                        
                        %% Pilot Sequences (unknown)
                        s_unk=zeros(kappa,L); % L pilot sequences.
                        for s_aux=1:L
                            if pilot_mod==0
                                s_unk(:,s_aux)=orth_vectors_mix(:,1); % Same pilot sequences.
                            else
                                s_unk(:,s_aux)=cons_sym_mix(:,1);
                            end
                        end
                        
                        %% Pilot matrix
                        S=kron(s(:,1),eye(M));
                        
                        H_est=zeros(M,U,L);
                        H=zeros(M,L,L,U);
                        for aux_L=1:L % Estimation of L desired channels.
                            
                            for K=1:U
                                
                                N=((1/sqrt(2))*sqrt(diag(sigman2))*randn(M,tao)+1i*(1/sqrt(2))*sqrt(diag(sigman2))*randn(M,tao)); % Spatially and temporally AWGN.
                                N_unk=((1/sqrt(2))*sqrt(diag(sigman2))*randn(M,kappa)+1i*(1/sqrt(2))*sqrt(diag(sigman2))*randn(M,kappa)); % Spatially and temporally AWGN.
                                
                                %% Realization of L channels
                                for l=1:L
                                    h(:,l)=sqrtm(R(:,:,aux_L,l,K))*((1/sqrt(2))*sqrt(1)*randn(M,1)+1i*(1/sqrt(2))*sqrt(1)*randn(M,1));
                                end
                                
                                %% Realization of L channels (CPA Estimator)
                                if comp_coor==1
                                    if same_Ptx==0
                                        if scenario == 1
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        elseif scenario == 2
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d_U_BS)/(lambda)))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        elseif scenario == 3
                                            alpha_delta=1*(10^((SNR_dB)/10))*((10^((34.53 + 38*log10(d_U_BS))/10)))*(mean(sigman2)/tao)*(M*tao)/M;
                                        else
                                            alpha_delta=1*(10^((SNR_dB)/10))*(d_U_BS^gamma)*(mean(sigman2)/tao)*(M*tao)/M; % Constant dependent on the prescribed average SNR.
                                        end
                                    else
                                        alpha_delta=Ptx;
                                    end
                                    
                                    alpha=zeros(TC,SP,L); % Path attenuation.
                                    for l=1:L
                                        d=distances_coor_gen(aux_L,l,K);
                                        if scenario == 1
                                            PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                            beta=alpha_delta/(10^(PL/10));
                                        elseif scenario == 2
                                            PL=20*log10((4*pi*1)/(lambda))+ 10*n_pl*log10((4*pi*d)/(lambda));
                                            beta=alpha_delta/(10^(PL/10));
                                        elseif scenario == 3
                                            PL = 34.53 + 38*log10(d);
                                            beta=alpha_delta/(10^(PL/10));
                                        else
                                            beta=alpha_delta/(d^gamma); % Distance-based path loss.
                                        end
                                        alpha(:,:,l)=(1/sqrt(2))*sqrt(beta)*randn(TC,SP)+1i*(1/sqrt(2))*sqrt(beta)*randn(TC,SP);
                                    end
                                    
                                    theta=zeros(TC,SP,L); % Random AOA.
                                    for l=1:L
                                        for aux_TC=1:TC
                                            for aux_SP=1:SP
                                                if AS_active==1
                                                    if strcmp(AOA_dist,'Gaussian')==1
                                                        theta(aux_TC,aux_SP,l)=(mean_AOA_coor_gen(K,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA*randn(1,1));
                                                    else
                                                        theta(aux_TC,aux_SP,l)=(mean_AOA_coor_gen(K,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA) + ((mean_AOA_coor_gen(K,round(1 + (SL-1)*rand(1,1)),l,aux_L)+sigma_theta_AOA)-(mean_AOA_coor_gen(K,round(1 + (SL-1)*rand(1,1)),l,aux_L)-sigma_theta_AOA)).*rand(1,1);
                                                    end
                                                else
                                                    theta(aux_TC,aux_SP,l)=mean_AOA_coor_gen(K,aux_TC,l,aux_L);
                                                end
                                            end
                                        end
                                    end
                                    
                                    h_coor=zeros(M,L); % L channel vectors.
                                    for l=1:L
                                        for aux_TC=1:TC
                                            for aux_SP=1:SP
                                                a=exp(-1i*2*pi*D/lambda.*((0:M-1).')*cos(theta(aux_TC,aux_SP,l))); % Steering vector.
                                                if BF==0
                                                    h_coor(:,l)=h_coor(:,l)+(a*alpha(aux_TC,aux_SP,l));
                                                else
                                                    h_coor(:,l)=h_coor(:,l)+sqrt(abs(BF_v'*a))*(a*alpha(aux_TC,aux_SP,l));
                                                end
                                            end
                                        end
                                        h_coor(:,l)=h_coor(:,l)/sqrt(TC*SP);
                                    end
                                else
                                    h_coor=h;
                                end
                                
                                %% Received signal
                                Y_hs=h(:,aux_L)*(s(:,aux_L).'); % Received signal at the base station without noise and without interference.
                                
                                Y=zeros(M,tao); % Received signal at the base station without noise.
                                for l=1:L
                                    Y=Y+h(:,l)*(s(:,l).');
                                end
                                n=reshape(N,M*tao,1); % vec(N)
                                Y=Y+N; % Received signal at the base station.
                                y=reshape(Y,M*tao,1); % vec(Y)
                                
                                % Received signal (CPA Estimator)
                                Y_hs_coor=h_coor(:,aux_L)*(s(:,aux_L).'); % Received signal at the base station without noise and without interference.
                                
                                Y_coor=zeros(M,tao); % Received signal at the base station without noise.
                                for l=1:L
                                    Y_coor=Y_coor+h_coor(:,l)*(s(:,l).');
                                end
                                Y_coor=Y_coor+N; % Received signal at the base station.
                                y_coor=reshape(Y_coor,M*tao,1); % vec(Y)
                                
                                %% Received signal (unknown) (CPA Estimator)
                                Y_hs_coor_mix=h_coor(:,aux_L)*(s_unk(:,aux_L).'); % Received signal at the base station without noise and without interference.
                                
                                Y_coor_mix2=zeros(M,kappa); % Received signal at the base station without noise.
                                for l=1:L
                                    Y_coor_mix2=Y_coor_mix2+h_coor(:,l)*(s_unk(:,l).');
                                end
                                Y_coor_mix2=Y_coor_mix2+N_unk; % Received signal at the base station.
                                Y_coor_mix=[Y_coor Y_coor_mix2];
                                y_coor_mix=reshape(Y_coor_mix,M*(tao+kappa),1); % vec(Y)
                                
                                %% LS Estimator
                                h1_est_LS=Y*conj(s(:,aux_L))/(s(:,aux_L).'*conj(s(:,aux_L)));
                                h1_est_LS_no_int=(Y_hs+N)*conj(s(:,aux_L))/(s(:,aux_L).'*conj(s(:,aux_L)));
                                
                                %% CB Estimator
                                h1_est_CB=R(:,:,aux_L,aux_L,K)/(diag(sigman2)+tao*sum_R(:,:,aux_L,K))*S'*y_coor;
                                h1_est_CB_no_int=R_coor(:,:,aux_L,aux_L,K)/(diag(sigman2)+tao*R_coor(:,:,aux_L,aux_L,K))*S'*(S*h_coor(:,aux_L)+n);
                                
                                %% CPA Estimator
                                h1_est_CPA=R_coor(:,:,aux_L,aux_L,K)/(diag(sigman2)+tao*sum_R_coor(:,:,aux_L,K))*S'*y_coor;
                                
                                %% Noise Variance Estimator
                                %%% Equal Noise Variance %%%
                                if different_sigman2==0
                                    est_sigman2_MM=sigman2(1);
                                    est_sigman2_LS=sigman2(1);
                                    est_sigman2_LS_PSD=sigman2(1);
                                    est_sigman2_ML=sigman2(1);
                                    crlb_unk_res=0;
                                    crlb_k_res=0;
                                    est_sigman2_unk_pilot_MM=sigman2(1);
                                    est_sigman2_unk_pilot_ML=sigman2(1);
                                    est_sigman2_unk_pilot_ML_alt=sigman2(1);
                                    crlb_unk_unk_pilot_res=0;
                                    crlb_k_unk_pilot_res=0;
                                    crlb_unk_unk_pilot_res_alt=0;
                                    crlb_k_unk_pilot_res_alt=0;
                                    est_sigman2_mix_pilot_MM=sigman2(1);
                                    est_sigman2_mix_pilot_ML=sigman2(1);
                                    est_sigman2_mix_pilot_ML_alt=sigman2(1);
                                    crlb_unk_mix_pilot_res=0;
                                    crlb_k_mix_pilot_res=0;
                                    crlb_unk_mix_pilot_res_alt=0;
                                    crlb_k_mix_pilot_res_alt=0;
                                    if noise_var_est==1
                                        
                                        %% Known pilot
                                                                               
                                        %%% MM Estimator
                                        est_sigman2_MM=((sum(abs(y_coor).^2)-tao*(trace(sum_R_coor(:,:,aux_L,K))))/(M*tao));
                                        
                                        %%% ML Estimator
                                        [U_eig, D_diag1]=eig(sum_R_coor(:,:,aux_L,K));
                                        D_diag=eig(sum_R_coor(:,:,aux_L,K));
                                        c_aux=((abs(y_coor'*S*U_eig).^2)/tao).';
                                        
                                        sigman2_aux=sigman2_initial_guess; % Initial guess
                                        conv_ML_aux=0;
                                        while(1)
                                            conv_ML_aux=conv_ML_aux+1;
                                            %%% First derivative
                                            a_1st=-M*tao/sigman2_aux;
                                            b_1st=(1/sigman2_aux^2)*((y_coor'*y_coor)-(1/tao)*(y_coor'*(S*S')*y_coor));
                                            c_1st=sum( (D_diag*tao + ((D_diag*tao).^2)/sigman2_aux + c_aux) ./ (D_diag*tao+sigman2_aux).^2 );
                                            der_1st=(a_1st+b_1st+c_1st);
                                            
                                            %%% Second derivative
                                            a_2nd=M*tao/sigman2_aux^2;
                                            b_2nd=(2/sigman2_aux^3)*((1/tao)*(y_coor'*(S*S')*y_coor)-y_coor'*y_coor);
                                            c_2nd=-sum( ((3*(D_diag*tao).^2)/sigman2_aux + ((D_diag*tao).^3)/sigman2_aux^2 + 2*D_diag*tao + 2*c_aux ) ./ (D_diag*tao+sigman2_aux).^3 );
                                            der_2nd=(a_2nd+b_2nd+c_2nd);
                                            
                                            est_sigman2_ML=(sigman2_aux-der_1st/der_2nd);
                                            
                                            if abs(est_sigman2_ML-sigman2_aux)<err_est_sigman2_ML
                                                conv_vec(1,MC_aux)=conv_ML_aux;
                                                break;
                                            else
                                                sigman2_aux=est_sigman2_ML;
                                            end
                                            
                                        end
                                        
                                        %%% CRLB (random and unknown channel)
                                        B_diag=eig(sum_R_coor(:,:,aux_L,K));
                                        
                                        eigen_aux=0;
                                        for l=1:M
                                            eigen_aux=eigen_aux+(1/(1+(B_diag(l)*tao/sigman2(1))))^2;
                                        end
                                        
                                        crlb_unk_res=sigman2(1)^2/(M*(tao-1)+eigen_aux);
                                        
                                        %%% CRLB (random and known channel)
                                        if cmp_known_channel==1
                                            crlb_k_res=(sigman2(1)^2)/(M*tao);
                                        end
                                        
                                        %% Unknown pilot
                                        
                                        %%% MM Estimator
                                        est_sigman2_unk_pilot_MM=((y_coor'*y_coor)/(M*tao))-((trace(sum_R_coor(:,:,aux_L,K))/M));
                                        
                                        %%% ML Estimator. EM Newton - Raphson
                                        sigman2_aux=sigman2_initial_guess; % Initial guess
                                        conv_ML_aux=0;
                                        while(1)
                                            conv_ML_aux=conv_ML_aux+1;
                                            sigman2_aux_prev=sigman2_aux;
                                            E_SS=0;
                                            E_c_aux=0;
                                            sum_pdf_yS=0;
                                            s_aux=zeros(tao,1);
                                            C=zeros(M,M);
                                            for m=1:M
                                                C(m,m)=1/(sigman2_aux+tao*B_diag(m));
                                            end
                                            for s1=1:M_cons
                                                s_aux(1)=ideal_cons_points(s1);
                                                for s2=1:M_cons
                                                    s_aux(2)=ideal_cons_points(s2);
                                                    for s3=1:M_cons
                                                        s_aux(3)=ideal_cons_points(s3);
                                                        for s4=1:M_cons
                                                            s_aux(4)=ideal_cons_points(s4);
                                                            for s5=1:M_cons
                                                                s_aux(5)=ideal_cons_points(s5);
                                                                %                                                                 for s6=1:M_cons
                                                                %                                                                     s_aux(6)=ideal_cons_points(s6);
                                                                %                                                                     for s7=1:M_cons
                                                                %                                                                         s_aux(7)=ideal_cons_points(s7);
                                                                %                                                                         for s8=1:M_cons
                                                                %                                                                             s_aux(8)=ideal_cons_points(s8);
                                                                S_aux=kron(s_aux,eye(M));
                                                                det_Cy=(sigman2_aux^(M*tao))*prod(1+((tao*B_diag)/(sigman2_aux)));
                                                                inv_Cy=(eye(M*tao,M*tao)/sigman2_aux)-((1/(sigman2_aux*tao))*(S_aux*S_aux'))+(1/tao)*S_aux*U_eig*C*U_eig'*S_aux';
                                                                pdf_y_S=( 1 / ((pi^(M*tao))*(det_Cy)) ) * exp (-y_coor'*inv_Cy*y_coor);
                                                                E_SS=E_SS+S_aux*S_aux'*pdf_y_S;
                                                                E_c_aux=E_c_aux+(((abs(y_coor'*S_aux*U_eig).^2)/tao).')*pdf_y_S;
                                                                sum_pdf_yS=sum_pdf_yS+pdf_y_S;
                                                                %                                                                         end
                                                                %                                                                     end
                                                                %                                                                 end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                            E_SS=E_SS/sum_pdf_yS;
                                            E_c_aux=E_c_aux/sum_pdf_yS;
                                            
                                            while(1)
                                                %%% First derivative
                                                a_1st=-M*tao/sigman2_aux;
                                                b_1st=(1/sigman2_aux^2)*((y_coor'*y_coor)-(1/tao)*(y_coor'*E_SS*y_coor));
                                                c_1st=sum( (D_diag*tao + ((D_diag*tao).^2)/sigman2_aux + E_c_aux) ./ (D_diag*tao+sigman2_aux).^2 );
                                                der_1st=(a_1st+b_1st+c_1st);
                                                
                                                %%% Second derivative
                                                a_2nd=M*tao/sigman2_aux^2;
                                                b_2nd=(2/sigman2_aux^3)*((1/tao)*(y_coor'*E_SS*y_coor)-y_coor'*y_coor);
                                                c_2nd=-sum( ((3*(D_diag*tao).^2)/sigman2_aux + ((D_diag*tao).^3)/sigman2_aux^2 + 2*D_diag*tao + 2*E_c_aux ) ./ (D_diag*tao+sigman2_aux).^3 );
                                                der_2nd=(a_2nd+b_2nd+c_2nd);
                                                
                                                sigman2_aux2=(sigman2_aux-der_1st/der_2nd);
                                                if isnan(sigman2_aux2)
                                                    break;
                                                end
                                                
                                                if abs(sigman2_aux2-sigman2_aux)<err_est_sigman2_ML
                                                    break;
                                                else
                                                    sigman2_aux=sigman2_aux2;
                                                end
                                                
                                            end
                                            if isnan(sigman2_aux2)
                                                sigman2_aux=sigman2_aux2;
                                                break;
                                            end
                                            sigman2_aux=sigman2_aux2;
                                            if abs(sigman2_aux-sigman2_aux_prev)<err_est_sigman2_ML
                                                conv_vec(2,MC_aux)=conv_ML_aux;
                                                break;
                                            end
                                            
                                        end
                                        est_sigman2_unk_pilot_ML=sigman2_aux;
                                        
                                        %%% ML Estimator (Alternative form). EM Newton - Raphson
                                        S_vec=zeros(M*tao,M,M_cons^tao);
                                        T_est_vec=zeros(1,M_cons^tao);
                                        S_vec_aux=1;
                                        s_aux=zeros(tao,1);
                                        for s1=1:M_cons
                                            s_aux(1)=ideal_cons_points(s1);
                                            for s2=1:M_cons
                                                s_aux(2)=ideal_cons_points(s2);
                                                for s3=1:M_cons
                                                    s_aux(3)=ideal_cons_points(s3);
                                                    for s4=1:M_cons
                                                        s_aux(4)=ideal_cons_points(s4);
                                                        for s5=1:M_cons
                                                            s_aux(5)=ideal_cons_points(s5);
                                                            %                                                                 for s6=1:M_cons
                                                            %                                                                     s_aux(6)=ideal_cons_points(s6);
                                                            %                                                                     for s7=1:M_cons
                                                            %                                                                         s_aux(7)=ideal_cons_points(s7);
                                                            %                                                                         for s8=1:M_cons
                                                            %                                                                             s_aux(8)=ideal_cons_points(s8);
                                                            S_aux=kron(s_aux,eye(M));
                                                            S_vec(:,:,S_vec_aux)=S_aux;
                                                            T_est_vec(S_vec_aux)=y_coor'*(S_aux*S_aux')*y_coor;
                                                            S_vec_aux=S_vec_aux+1;
                                                            %                                                                         end
                                                            %                                                                     end
                                                            %                                                                 end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                        [~,max_i]=max(T_est_vec);
                                        S_max=S_vec(:,:,max_i);
                                        E_SS=S_max*S_max';
                                        E_c_aux=(((abs(y_coor'*S_max*U_eig).^2)/tao).');
                                        
                                        sigman2_aux=sigman2_initial_guess; % Initial guess
                                        while(1)
                                            %%% First derivative
                                            a_1st=-M*tao/sigman2_aux;
                                            b_1st=(1/sigman2_aux^2)*((y_coor'*y_coor)-(1/tao)*(y_coor'*E_SS*y_coor));
                                            c_1st=sum( (D_diag*tao + ((D_diag*tao).^2)/sigman2_aux + E_c_aux) ./ (D_diag*tao+sigman2_aux).^2 );
                                            der_1st=(a_1st+b_1st+c_1st);
                                            
                                            %%% Second derivative
                                            a_2nd=M*tao/sigman2_aux^2;
                                            b_2nd=(2/sigman2_aux^3)*((1/tao)*(y_coor'*E_SS*y_coor)-y_coor'*y_coor);
                                            c_2nd=-sum( ((3*(D_diag*tao).^2)/sigman2_aux + ((D_diag*tao).^3)/sigman2_aux^2 + 2*D_diag*tao + 2*E_c_aux ) ./ (D_diag*tao+sigman2_aux).^3 );
                                            der_2nd=(a_2nd+b_2nd+c_2nd);
                                            
                                            est_sigman2_unk_pilot_ML_alt=(sigman2_aux-der_1st/der_2nd);
                                            
                                            if abs(est_sigman2_unk_pilot_ML_alt-sigman2_aux)<err_est_sigman2_ML
                                                break;
                                            else
                                                sigman2_aux=est_sigman2_unk_pilot_ML_alt;
                                            end
                                            
                                        end
                                        
                                        %%% CRLB (random and unknown channel)
                                        C=zeros(M,M);
                                        for m=1:M
                                            C(m,m)=1/(sigman2(1)+tao*B_diag(m));
                                        end
                                        sum_T=0;
                                        sum_TD=0;
                                        sum_der=0;
                                        D_est_der_vec=zeros(1,M_cons^tao);
                                        T_est_vec=zeros(1,M_cons^tao);
                                        D_est_der_vec_aux=1;
                                        s_aux=zeros(tao,1);
                                        for s1=1:M_cons
                                            s_aux(1)=ideal_cons_points(s1);
                                            for s2=1:M_cons
                                                s_aux(2)=ideal_cons_points(s2);
                                                for s3=1:M_cons
                                                    s_aux(3)=ideal_cons_points(s3);
                                                    for s4=1:M_cons
                                                        s_aux(4)=ideal_cons_points(s4);
                                                        for s5=1:M_cons
                                                            s_aux(5)=ideal_cons_points(s5);
                                                            %                                                             for s6=1:M_cons
                                                            %                                                                 s_aux(6)=ideal_cons_points(s6);
                                                            %                                                                 for s7=1:M_cons
                                                            %                                                                     s_aux(7)=ideal_cons_points(s7);
                                                            %                                                                     for s8=1:M_cons
                                                            %                                                                         s_aux(8)=ideal_cons_points(s8);
                                                            S_aux=kron(s_aux,eye(M));
                                                            c_aux=((abs(y_coor'*S_aux*U_eig).^2)/tao).';
                                                            T_est=exp((1/(sigman2(1)*tao))*y_coor'*(S_aux*S_aux')*y_coor-(1/tao)*y_coor'*S_aux*U_eig*C*U_eig'*S_aux'*y_coor);
                                                            D_est=sum((c_aux)./(sigman2(1)+B_diag*tao).^2)-(y_coor'*(S_aux*S_aux')*y_coor)/(tao*sigman2(1)^2);
                                                            D_est_der=sum((-2*c_aux)./(sigman2(1)+B_diag*tao).^3)+(2*y_coor'*(S_aux*S_aux')*y_coor)/(tao*sigman2(1)^3);
                                                            sum_T=sum_T+T_est;
                                                            sum_TD=sum_TD+T_est*D_est;
                                                            sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                            D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                            T_est_vec(D_est_der_vec_aux)=y_coor'*(S_aux*S_aux')*y_coor;
                                                            D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                            %                                                                     end
                                                            %                                                                 end
                                                            %                                                             end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                        der2=(sum_der/sum_T)-(((sum_TD))/((sum_T)))^2;
                                        [~,max_i]=max(T_est_vec);
                                        der2_alt=D_est_der_vec(max_i);
                                        crlb_unk_unk_pilot_res=((M*tao)/(sigman2(1)^2))-sum((((B_diag*tao).^3/(sigman2(1)^2))+(3*(B_diag*tao).^2/sigman2(1))+2*B_diag*tao)./(sigman2(1)+B_diag*tao).^3)-(2*(y_coor'*y_coor)/(sigman2(1)^3))+der2;
                                        crlb_unk_unk_pilot_res_alt=((M*tao)/(sigman2(1)^2))-sum((((B_diag*tao).^3/(sigman2(1)^2))+(3*(B_diag*tao).^2/sigman2(1))+2*B_diag*tao)./(sigman2(1)+B_diag*tao).^3)-(2*(y_coor'*y_coor)/(sigman2(1)^3))+der2_alt;
                                        
                                        %%% CRLB (random and known channel)
                                        if cmp_known_channel==1
                                            B_diag_aux=B_diag;
                                            B_diag=eig(zeros(M,M));
                                            C=zeros(M,M);
                                            for m=1:M
                                                C(m,m)=1/(sigman2(1)+tao*B_diag(m));
                                            end
                                            sum_T=0;
                                            sum_TD=0;
                                            sum_der=0;
                                            D_est_der_vec=zeros(1,M_cons^tao);
                                            T_est_vec=zeros(1,M_cons^tao);
                                            D_est_der_vec_aux=1;
                                            s_aux=zeros(tao,1);
                                            for s1=1:M_cons
                                                s_aux(1)=ideal_cons_points(s1);
                                                for s2=1:M_cons
                                                    s_aux(2)=ideal_cons_points(s2);
                                                    for s3=1:M_cons
                                                        s_aux(3)=ideal_cons_points(s3);
                                                        for s4=1:M_cons
                                                            s_aux(4)=ideal_cons_points(s4);
                                                            for s5=1:M_cons
                                                                s_aux(5)=ideal_cons_points(s5);
                                                            %                                                             for s6=1:M_cons
                                                            %                                                                 s_aux(6)=ideal_cons_points(s6);
                                                            %                                                                 for s7=1:M_cons
                                                            %                                                                     s_aux(7)=ideal_cons_points(s7);
                                                            %                                                                     for s8=1:M_cons
                                                            %                                                                         s_aux(8)=ideal_cons_points(s8);
                                                            S_aux=kron(s_aux,eye(M));
                                                            c_aux=((abs(y_coor'*S_aux*U_eig).^2)/tao).';
                                                            T_est=exp((1/(sigman2(1)*tao))*y_coor'*(S_aux*S_aux')*y_coor-(1/tao)*y_coor'*S_aux*U_eig*C*U_eig'*S_aux'*y_coor);
                                                            D_est=sum((c_aux)./(sigman2(1)+B_diag*tao).^2)-(y_coor'*(S_aux*S_aux')*y_coor)/(tao*sigman2(1)^2);
                                                            D_est_der=sum((-2*c_aux)./(sigman2(1)+B_diag*tao).^3)+(2*y_coor'*(S_aux*S_aux')*y_coor)/(tao*sigman2(1)^3);
                                                            sum_T=sum_T+T_est;
                                                            sum_TD=sum_TD+T_est*D_est;
                                                            sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                            D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                            T_est_vec(D_est_der_vec_aux)=y_coor'*(S_aux*S_aux')*y_coor;
                                                            D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                            %                                                                     end
                                                            %                                                                 end
                                                            %                                                             end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                            der2=(sum_der/sum_T)-(((sum_TD))/((sum_T)))^2;
                                            [~,max_i]=max(T_est_vec);
                                            der2_alt=D_est_der_vec(max_i);
                                            crlb_k_unk_pilot_res=((M*tao)/(sigman2(1)^2))-sum((((B_diag*tao).^3/(sigman2(1)^2))+(3*(B_diag*tao).^2/sigman2(1))+2*B_diag*tao)./(sigman2(1)+B_diag*tao).^3)-(2*(y_coor'*y_coor)/(sigman2(1)^3))+der2;
                                            crlb_k_unk_pilot_res_alt=((M*tao)/(sigman2(1)^2))-sum((((B_diag*tao).^3/(sigman2(1)^2))+(3*(B_diag*tao).^2/sigman2(1))+2*B_diag*tao)./(sigman2(1)+B_diag*tao).^3)-(2*(y_coor'*y_coor)/(sigman2(1)^3))+der2_alt;
                                            B_diag=B_diag_aux;
                                        end
                                        
                                        %% Mixed pilot
                                        %%% MM Estimator
                                        est_sigman2_mix_pilot_MM=((y_coor_mix'*y_coor_mix)/(M*(tao+kappa)))-((trace(sum_R_coor(:,:,aux_L,K))/M));
                                        
                                        %%% ML Estimator. EM Newton - Raphson
                                        sigman2_aux=sigman2_initial_guess; % Initial guess
                                        conv_ML_aux=0;
                                        while(1)
                                            conv_ML_aux=conv_ML_aux+1;
                                            sigman2_aux_prev=sigman2_aux;
                                            E_SS=0;
                                            E_c_aux=0;
                                            sum_pdf_yS=0;
                                            s_aux=zeros((tao+kappa),1);
                                            s_aux(1:tao)=s(1:tao,aux_L);
                                            C=zeros(M,M);
                                            for m=1:M
                                                C(m,m)=1/(sigman2_aux+(tao+kappa)*B_diag(m));
                                            end
                                            for sunk_1=1:M_cons
                                                s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                for sunk_2=1:M_cons
                                                    s_aux(tao+2)=ideal_cons_points(sunk_2);
                                                    %                                                     for sunk_3=1:M_cons
                                                    %                                                         s_aux(tao+3)=ideal_cons_points(sunk_3);
                                                    %                                                         for sunk_4=1:M_cons
                                                    %                                                             s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                    %                                                                                                                 for sunk_5=1:M_cons
                                                    %                                                                                                                     s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                    %                                                     %                                                                 for sunk_6=1:M_cons
                                                    %                                                                     s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                    %                                                                     for sunk_7=1:M_cons
                                                    %                                                                         s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                    %                                                                         for sunk_8=1:M_cons
                                                    %                                                                             s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                    S_aux=kron(s_aux,eye(M));
                                                    det_Cy=(sigman2_aux^(M*(tao+kappa)))*prod(1+(((tao+kappa)*B_diag)/(sigman2_aux)));
                                                    inv_Cy=(eye(M*(tao+kappa),M*(tao+kappa))/sigman2_aux)-((1/(sigman2_aux*(tao+kappa)))*(S_aux*S_aux'))+(1/(tao+kappa))*S_aux*U_eig*C*U_eig'*S_aux';
                                                    pdf_y_S=( 1 / ((pi^(M*(tao+kappa)))*(det_Cy)) ) * exp (-y_coor_mix'*inv_Cy*y_coor_mix);
                                                    E_SS=E_SS+S_aux*S_aux'*pdf_y_S;
                                                    E_c_aux=E_c_aux+(((abs(y_coor_mix'*S_aux*U_eig).^2)/(tao+kappa)).')*pdf_y_S;
                                                    sum_pdf_yS=sum_pdf_yS+pdf_y_S;
                                                    %                                                                         end
                                                    %                                                                     end
                                                    %                                                                 end
                                                    %                                                                                                                 end
                                                    %                                                         end
                                                    %                                                     end
                                                end
                                            end
                                            E_SS=E_SS/sum_pdf_yS;
                                            E_c_aux=E_c_aux/sum_pdf_yS;
                                            
                                            while(1)
                                                %%% First derivative
                                                a_1st=-M*(tao+kappa)/sigman2_aux;
                                                b_1st=(1/sigman2_aux^2)*((y_coor_mix'*y_coor_mix)-(1/(tao+kappa))*(y_coor_mix'*E_SS*y_coor_mix));
                                                c_1st=sum( (D_diag*(tao+kappa) + ((D_diag*(tao+kappa)).^2)/sigman2_aux + E_c_aux) ./ (D_diag*(tao+kappa)+sigman2_aux).^2 );
                                                der_1st=(a_1st+b_1st+c_1st);
                                                
                                                %%% Second derivative
                                                a_2nd=M*(tao+kappa)/sigman2_aux^2;
                                                b_2nd=(2/sigman2_aux^3)*((1/(tao+kappa))*(y_coor_mix'*E_SS*y_coor_mix)-y_coor_mix'*y_coor_mix);
                                                c_2nd=-sum( ((3*(D_diag*(tao+kappa)).^2)/sigman2_aux + ((D_diag*(tao+kappa)).^3)/sigman2_aux^2 + 2*D_diag*(tao+kappa) + 2*E_c_aux ) ./ (D_diag*(tao+kappa)+sigman2_aux).^3 );
                                                der_2nd=(a_2nd+b_2nd+c_2nd);
                                                
                                                sigman2_aux2=(sigman2_aux-der_1st/der_2nd);
                                                if isnan(sigman2_aux2)
                                                    break;
                                                end
                                                
                                                if abs(sigman2_aux2-sigman2_aux)<err_est_sigman2_ML
                                                    break;
                                                else
                                                    sigman2_aux=sigman2_aux2;
                                                end
                                                
                                            end
                                            if isnan(sigman2_aux2)
                                                sigman2_aux=sigman2_aux2;
                                                break;
                                            end
                                            sigman2_aux=sigman2_aux2;
                                            if abs(sigman2_aux-sigman2_aux_prev)<err_est_sigman2_ML
                                                conv_vec(2,MC_aux)=conv_ML_aux;
                                                break;
                                            end
                                            
                                        end
                                        est_sigman2_mix_pilot_ML=sigman2_aux;
                                        
                                        %%% ML Estimator (Alternative form). EM Newton - Raphson
                                        S_vec=zeros(M*(tao+kappa),M,M_cons^kappa);
                                        T_est_vec=zeros(1,M_cons^kappa);
                                        S_vec_aux=1;
                                        s_aux=zeros((tao+kappa),1);
                                        s_aux(1:tao)=s(1:tao,aux_L);
                                        for sunk_1=1:M_cons
                                            s_aux(tao+1)=ideal_cons_points(sunk_1);
                                            for sunk_2=1:M_cons
                                                s_aux(tao+2)=ideal_cons_points(sunk_2);
                                                %                                                 for sunk_3=1:M_cons
                                                %                                                     s_aux(tao+3)=ideal_cons_points(sunk_3);
                                                %                                                     for sunk_4=1:M_cons
                                                %                                                         s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                %                                                                                                                 for sunk_5=1:M_cons
                                                %                                                                                                                     s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                %                                                     %                                                                 for sunk_6=1:M_cons
                                                %                                                                     s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                %                                                                     for sunk_7=1:M_cons
                                                %                                                                         s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                %                                                                         for sunk_8=1:M_cons
                                                %                                                                             s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                
                                                S_aux=kron(s_aux,eye(M));
                                                S_vec(:,:,S_vec_aux)=S_aux;
                                                T_est_vec(S_vec_aux)=y_coor_mix'*(S_aux*S_aux')*y_coor_mix;
                                                S_vec_aux=S_vec_aux+1;
                                                %                                                                         end
                                                %                                                                     end
                                                %                                                                 end
                                                %                                                             end
                                                %                                                     end
                                                %                                                 end
                                            end
                                        end
                                        [~,max_i]=max(T_est_vec);
                                        S_max=S_vec(:,:,max_i);
                                        E_SS=S_max*S_max';
                                        E_c_aux=((abs(y_coor_mix'*S_max*U_eig).^2)/(tao+kappa)).';
                                        
                                        sigman2_aux=sigman2_initial_guess; % Initial guess
                                        while(1)
                                            %%% First derivative
                                            a_1st=-M*(tao+kappa)/sigman2_aux;
                                            b_1st=(1/sigman2_aux^2)*((y_coor_mix'*y_coor_mix)-(1/(tao+kappa))*(y_coor_mix'*E_SS*y_coor_mix));
                                            c_1st=sum( (D_diag*(tao+kappa) + ((D_diag*(tao+kappa)).^2)/sigman2_aux + E_c_aux) ./ (D_diag*(tao+kappa)+sigman2_aux).^2 );
                                            der_1st=(a_1st+b_1st+c_1st);
                                            
                                            %%% Second derivative
                                            a_2nd=M*(tao+kappa)/sigman2_aux^2;
                                            b_2nd=(2/sigman2_aux^3)*((1/(tao+kappa))*(y_coor_mix'*E_SS*y_coor_mix)-y_coor_mix'*y_coor_mix);
                                            c_2nd=-sum( ((3*(D_diag*(tao+kappa)).^2)/sigman2_aux + ((D_diag*(tao+kappa)).^3)/sigman2_aux^2 + 2*D_diag*(tao+kappa) + 2*E_c_aux ) ./ (D_diag*(tao+kappa)+sigman2_aux).^3 );
                                            der_2nd=(a_2nd+b_2nd+c_2nd);
                                            
                                            est_sigman2_mix_pilot_ML_alt=(sigman2_aux-der_1st/der_2nd);
                                            
                                            if abs(est_sigman2_mix_pilot_ML_alt-sigman2_aux)<err_est_sigman2_ML
                                                break;
                                            else
                                                sigman2_aux=est_sigman2_mix_pilot_ML_alt;
                                            end
                                            
                                        end
                                        
                                        %%% CRLB (random and unknown channel)
                                        C=zeros(M,M);
                                        for m=1:M
                                            C(m,m)=1/(sigman2(1)+(tao+kappa)*B_diag(m));
                                        end
                                        sum_T=0;
                                        sum_TD=0;
                                        sum_der=0;
                                        D_est_der_vec=zeros(1,M_cons^kappa);
                                        T_est_vec=zeros(1,M_cons^kappa);
                                        D_est_der_vec_aux=1;
                                        s_aux=zeros((tao+kappa),1);
                                        s_aux(1:tao)=s(1:tao,aux_L);
                                        for sunk_1=1:M_cons
                                            s_aux(tao+1)=ideal_cons_points(sunk_1);
                                            for sunk_2=1:M_cons
                                                s_aux(tao+2)=ideal_cons_points(sunk_2);
                                                %                                                 for sunk_3=1:M_cons
                                                %                                                     s_aux(tao+3)=ideal_cons_points(sunk_3);
                                                %                                                     for sunk_4=1:M_cons
                                                %                                                         s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                %                                                                                                         for sunk_5=1:M_cons
                                                %                                                                                                             s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                %                                                                                                              for sunk_6=1:M_cons
                                                %                                                                 s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                %                                                                 for sunk_7=1:M_cons
                                                %                                                                     s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                %                                                                     for sunk_8=1:M_cons
                                                %                                                                         s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                S_aux=kron(s_aux,eye(M));
                                                c_aux=((abs(y_coor_mix'*S_aux*U_eig).^2)/(tao+kappa)).';
                                                T_est=exp((1/(sigman2(1)*(tao+kappa)))*y_coor_mix'*(S_aux*S_aux')*y_coor_mix-(1/(tao+kappa))*y_coor_mix'*S_aux*U_eig*C*U_eig'*S_aux'*y_coor_mix);
                                                D_est=sum((c_aux)./(sigman2(1)+B_diag*(tao+kappa)).^2)-(y_coor_mix'*(S_aux*S_aux')*y_coor_mix)/((tao+kappa)*sigman2(1)^2);
                                                D_est_der=sum((-2*c_aux)./(sigman2(1)+B_diag*(tao+kappa)).^3)+(2*y_coor_mix'*(S_aux*S_aux')*y_coor_mix)/((tao+kappa)*sigman2(1)^3);
                                                sum_T=sum_T+T_est;
                                                sum_TD=sum_TD+T_est*D_est;
                                                sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                T_est_vec(D_est_der_vec_aux)=y_coor_mix'*(S_aux*S_aux')*y_coor_mix;
                                                D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                %                                                                     end
                                                %                                                                 end
                                                %                                                             end
                                                %                                                                                                         end
                                                %                                                     end
                                                %                                                 end
                                            end
                                        end
                                        der2=(sum_der/sum_T)-(((sum_TD))/((sum_T)))^2;
                                        [~,max_i]=max(T_est_vec);
                                        der2_alt=D_est_der_vec(max_i);
                                        crlb_unk_mix_pilot_res=((M*(tao+kappa))/(sigman2(1)^2))-sum((((B_diag*(tao+kappa)).^3/(sigman2(1)^2))+(3*(B_diag*(tao+kappa)).^2/sigman2(1))+2*B_diag*(tao+kappa))./(sigman2(1)+B_diag*(tao+kappa)).^3)-(2*(y_coor_mix'*y_coor_mix)/(sigman2(1)^3))+der2;
                                        crlb_unk_mix_pilot_res_alt=((M*(tao+kappa))/(sigman2(1)^2))-sum((((B_diag*(tao+kappa)).^3/(sigman2(1)^2))+(3*(B_diag*(tao+kappa)).^2/sigman2(1))+2*B_diag*(tao+kappa))./(sigman2(1)+B_diag*(tao+kappa)).^3)-(2*(y_coor_mix'*y_coor_mix)/(sigman2(1)^3))+der2_alt;
                                        
                                        
                                        %%% CRLB (random and known channel)
                                        if cmp_known_channel==1
                                            B_diag_aux=B_diag;
                                            B_diag=eig(zeros(M,M));
                                            C=zeros(M,M);
                                            for m=1:M
                                                C(m,m)=1/(sigman2(1)+(tao+kappa)*B_diag(m));
                                            end
                                            sum_T=0;
                                            sum_TD=0;
                                            sum_der=0;
                                            D_est_der_vec=zeros(1,M_cons^kappa);
                                            T_est_vec=zeros(1,M_cons^kappa);
                                            D_est_der_vec_aux=1;
                                            s_aux=zeros((tao+kappa),1);
                                            s_aux(1:tao)=s(1:tao,aux_L);
                                            for sunk_1=1:M_cons
                                                s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                for sunk_2=1:M_cons
                                                    s_aux(tao+2)=ideal_cons_points(sunk_2);
                                                    %                                                     for sunk_3=1:M_cons
                                                    %                                                         s_aux(tao+3)=ideal_cons_points(sunk_3);
                                                    %                                                         for sunk_4=1:M_cons
                                                    %                                                             s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                    %                                                         for sunk_5=1:M_cons
                                                    %                                                             s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                    %                                                             for sunk_6=1:M_cons
                                                    %                                                                 s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                    %                                                                 for sunk_7=1:M_cons
                                                    %                                                                     s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                    %                                                                     for sunk_8=1:M_cons
                                                    %                                                                         s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                    S_aux=kron(s_aux,eye(M));
                                                    c_aux=((abs(y_coor_mix'*S_aux*U_eig).^2)/(tao+kappa)).';
                                                    T_est=exp((1/(sigman2(1)*(tao+kappa)))*y_coor_mix'*(S_aux*S_aux')*y_coor_mix-(1/(tao+kappa))*y_coor_mix'*S_aux*U_eig*C*U_eig'*S_aux'*y_coor_mix);
                                                    D_est=sum((c_aux)./(sigman2(1)+B_diag*(tao+kappa)).^2)-(y_coor_mix'*(S_aux*S_aux')*y_coor_mix)/((tao+kappa)*sigman2(1)^2);
                                                    D_est_der=sum((-2*c_aux)./(sigman2(1)+B_diag*(tao+kappa)).^3)+(2*y_coor_mix'*(S_aux*S_aux')*y_coor_mix)/((tao+kappa)*sigman2(1)^3);
                                                    sum_T=sum_T+T_est;
                                                    sum_TD=sum_TD+T_est*D_est;
                                                    sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                    D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                    T_est_vec(D_est_der_vec_aux)=y_coor_mix'*(S_aux*S_aux')*y_coor_mix;
                                                    D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                    %                                                                     end
                                                    %                                                                 end
                                                    %                                                             end
                                                    %                                                         end
                                                    %                                                         end
                                                    %                                                     end
                                                end
                                            end
                                            der2=(sum_der/sum_T)-(((sum_TD))/((sum_T)))^2;
                                            [~,max_i]=max(T_est_vec);
                                            der2_alt=D_est_der_vec(max_i);
                                            crlb_k_mix_pilot_res=((M*(tao+kappa))/(sigman2(1)^2))-sum((((B_diag*(tao+kappa)).^3/(sigman2(1)^2))+(3*(B_diag*(tao+kappa)).^2/sigman2(1))+2*B_diag*(tao+kappa))./(sigman2(1)+B_diag*(tao+kappa)).^3)-(2*(y_coor_mix'*y_coor_mix)/(sigman2(1)^3))+der2;
                                            crlb_k_mix_pilot_res_alt=((M*(tao+kappa))/(sigman2(1)^2))-sum((((B_diag*(tao+kappa)).^3/(sigman2(1)^2))+(3*(B_diag*(tao+kappa)).^2/sigman2(1))+2*B_diag*(tao+kappa))./(sigman2(1)+B_diag*(tao+kappa)).^3)-(2*(y_coor_mix'*y_coor_mix)/(sigman2(1)^3))+der2_alt;
                                            B_diag=B_diag_aux;
                                        end
                                        
                                        
                                        noise_est_vec_ML(noise_est_vec_ML_aux)=est_sigman2_ML;
                                        noise_est_vec_MM(noise_est_vec_MM_aux)=est_sigman2_MM;
                                        noise_est_vec_MM_aux=noise_est_vec_MM_aux+1;
                                        noise_est_vec_ML_aux=noise_est_vec_ML_aux+1;
                                    end
                                    
                                    est_sigman2_MM=est_sigman2_MM*ones(1,M);
                                    est_sigman2_LS=est_sigman2_LS*ones(1,M);
                                    est_sigman2_LS_PSD=est_sigman2_LS_PSD*ones(1,M);
                                    est_sigman2_ML=est_sigman2_ML*ones(1,M);
                                    crlb_unk_res=crlb_unk_res*ones(1,M);
                                    crlb_k_res=crlb_k_res*ones(1,M);
                                    est_sigman2_unk_pilot_MM=est_sigman2_unk_pilot_MM*ones(1,M);
                                    est_sigman2_unk_pilot_ML=est_sigman2_unk_pilot_ML*ones(1,M);
                                    est_sigman2_unk_pilot_ML_alt=est_sigman2_unk_pilot_ML_alt*ones(1,M);
                                    crlb_unk_unk_pilot_res=crlb_unk_unk_pilot_res*ones(1,M);
                                    crlb_k_unk_pilot_res=crlb_k_unk_pilot_res*ones(1,M);
                                    crlb_unk_unk_pilot_res_alt=crlb_unk_unk_pilot_res_alt*ones(1,M);
                                    crlb_k_unk_pilot_res_alt=crlb_k_unk_pilot_res_alt*ones(1,M);
                                    est_sigman2_mix_pilot_MM=est_sigman2_mix_pilot_MM*ones(1,M);
                                    est_sigman2_mix_pilot_ML=est_sigman2_mix_pilot_ML*ones(1,M);
                                    est_sigman2_mix_pilot_ML_alt=est_sigman2_mix_pilot_ML_alt*ones(1,M);
                                    crlb_unk_mix_pilot_res=crlb_unk_mix_pilot_res*ones(1,M);
                                    crlb_k_mix_pilot_res=crlb_k_mix_pilot_res*ones(1,M);
                                    crlb_unk_mix_pilot_res_alt=crlb_unk_mix_pilot_res_alt*ones(1,M);
                                    crlb_k_mix_pilot_res_alt=crlb_k_mix_pilot_res_alt*ones(1,M);
                                    
                                else
                                    
                                    est_sigman2_MM=sigman2;
                                    est_sigman2_LS=sigman2;
                                    est_sigman2_LS_PSD=sigman2;
                                    est_sigman2_ML=sigman2;
                                    crlb_unk_res=zeros(1,M);
                                    crlb_k_res=zeros(1,M);
                                    est_sigman2_unk_pilot_MM=sigman2;
                                    est_sigman2_unk_pilot_ML=sigman2;
                                    est_sigman2_unk_pilot_ML_alt=sigman2;
                                    crlb_unk_unk_pilot_res=zeros(1,M);
                                    crlb_k_unk_pilot_res=zeros(1,M);
                                    crlb_unk_unk_pilot_res_alt=zeros(1,M);
                                    crlb_k_unk_pilot_res_alt=zeros(1,M);
                                    est_sigman2_mix_pilot_MM=sigman2;
                                    est_sigman2_mix_pilot_ML=sigman2;
                                    est_sigman2_mix_pilot_ML_alt=sigman2;
                                    crlb_unk_mix_pilot_res=zeros(1,M);
                                    crlb_k_mix_pilot_res=zeros(1,M);
                                    crlb_unk_mix_pilot_res_alt=zeros(1,M);
                                    crlb_k_mix_pilot_res_alt=zeros(1,M);
                                    
                                    %%% Different Noise Variance %%%
                                    if noise_var_est==1
                                        
                                        for m=1:M
                                            
                                            y_m=Y_coor(m,:).';
                                            y_mix_m=Y_coor_mix(m,:).';
                                            
                                            %% Known pilot
                                            
                                            %%% MM Estimator
                                            est_sigman2_MM(m)=((y_m'*y_m)/(tao))-(sum_R_coor(m,m,aux_L,K));
                                            
                                            %%% ML Estimator
                                            sigman2_aux=sigman2_initial_guess; % Initial guess
                                            conv_ML_aux=0;
                                            while(1)
                                                conv_ML_aux=conv_ML_aux+1;
                                                %%% First derivative
                                                a_1st=(1/sigman2_aux^2)*(y_m'*y_m);
                                                b_1st=(((1/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-((1/tao)/(sigman2_aux^2)))*y_m'*(s(:,aux_L)*s(:,aux_L)')*y_m;
                                                c_1st=(((tao*sum_R_coor(m,m,aux_L,K))+((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-(tao/sigman2_aux);
                                                der_1st=(a_1st+b_1st+c_1st);
                                                
                                                %%% Second derivative
                                                a_2nd=-(2/sigman2_aux^3)*(y_m'*y_m);
                                                b_2nd=(((2/tao)/(sigman2_aux^3))-((2/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3))*y_m'*(s(:,aux_L)*s(:,aux_L)')*y_m;
                                                c_2nd=-((((tao*sum_R_coor(m,m,aux_L,K))^3/(sigman2_aux^2))+3*((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux))+(2*tao*sum_R_coor(m,m,aux_L,K)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3)+(tao/(sigman2_aux^2));
                                                der_2nd=(a_2nd+b_2nd+c_2nd);
                                                
                                                est_sigman2_ML(m)=(sigman2_aux-der_1st/der_2nd);
                                                
                                                if abs(est_sigman2_ML(m)-sigman2_aux)<err_est_sigman2_ML
                                                    conv_vec(1,MC_aux)=conv_ML_aux;
                                                    break;
                                                else
                                                    sigman2_aux=est_sigman2_ML(m);
                                                end
                                                
                                            end
                                            
                                            %%% CRLB (random and unknown channel)
                                            crlb_unk_res(m)=(sigman2(m)^2)/(tao-1+((1)/(1+((tao*sum_R_coor(m,m,aux_L,K))/sigman2(m)))^2));
                                            
                                            %%% CRLB (random and known channel)
                                            if cmp_known_channel==1
                                                crlb_k_res(m)=((sigman2(m)^2)/tao);
                                            end
                                            
                                            %% Unknown pilot
                                            
                                            %%% MM Estimator
                                            est_sigman2_unk_pilot_MM(m)=((y_m'*y_m/tao))-sum_R_coor(m,m,aux_L,K);
                                            
                                            %%% ML Estimator. EM Newton - Raphson
                                            sigman2_aux=sigman2_initial_guess; % Initial guess
                                            conv_ML_aux=0;
                                            while(1)
                                                conv_ML_aux=conv_ML_aux+1;
                                                sigman2_aux_prev=sigman2_aux;
                                                E_SS=0;
                                                sum_pdf_yS=0;
                                                s_aux=zeros(tao,1);
                                                for s1=1:M_cons
                                                    s_aux(1)=ideal_cons_points(s1);
                                                    for s2=1:M_cons
                                                        s_aux(2)=ideal_cons_points(s2);
                                                        for s3=1:M_cons
                                                            s_aux(3)=ideal_cons_points(s3);
                                                            for s4=1:M_cons
                                                                s_aux(4)=ideal_cons_points(s4);
                                                                for s5=1:M_cons
                                                                    s_aux(5)=ideal_cons_points(s5);
%                                                                     for s6=1:M_cons
%                                                                         s_aux(6)=ideal_cons_points(s6);
%                                                                         for s7=1:M_cons
%                                                                             s_aux(7)=ideal_cons_points(s7);
%                                                                             for s8=1:M_cons
%                                                                                 s_aux(8)=ideal_cons_points(s8);
                                                                                det_Cy=(sigman2_aux)^tao+tao*sum_R_coor(m,m,aux_L,K)*(sigman2_aux)^(tao-1);
                                                                                inv_Cy=(1/sigman2_aux)*((eye(tao,tao))-(s_aux*s_aux')/((sigman2_aux/sum_R_coor(m,m,aux_L,K))+tao));
                                                                                pdf_y_S=(1/((pi^tao)*det_Cy))*exp(-y_m'*inv_Cy*y_m);
                                                                                E_SS=E_SS+s_aux*s_aux'*pdf_y_S;
                                                                                sum_pdf_yS=sum_pdf_yS+pdf_y_S;
%                                                                             end
%                                                                         end
%                                                                     end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                                E_SS=E_SS/sum_pdf_yS;
                                                
                                                while(1)
                                                    %%% First derivative
                                                    a_1st=(1/sigman2_aux^2)*(y_m'*y_m);
                                                    b_1st=(((1/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-((1/tao)/(sigman2_aux^2)))*y_m'*(E_SS)*y_m;
                                                    c_1st=(((tao*sum_R_coor(m,m,aux_L,K))+((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-(tao/sigman2_aux);
                                                    der_1st=(a_1st+b_1st+c_1st);
                                                    
                                                    %%% Second derivative
                                                    a_2nd=-(2/sigman2_aux^3)*(y_m'*y_m);
                                                    b_2nd=(((2/tao)/(sigman2_aux^3))-((2/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3))*y_m'*(E_SS)*y_m;
                                                    c_2nd=-((((tao*sum_R_coor(m,m,aux_L,K))^3/(sigman2_aux^2))+3*((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux))+(2*tao*sum_R_coor(m,m,aux_L,K)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3)+(tao/(sigman2_aux^2));
                                                    der_2nd=(a_2nd+b_2nd+c_2nd);
                                                    
                                                    sigman2_aux2=(sigman2_aux-der_1st/der_2nd);
                                                    if isnan(sigman2_aux2)
                                                        break;
                                                    end
                                                    
                                                    if abs(sigman2_aux2-sigman2_aux)<err_est_sigman2_ML
                                                        break;
                                                    else
                                                        sigman2_aux=sigman2_aux2;
                                                    end
                                                    
                                                end
                                                if isnan(sigman2_aux2)
                                                    sigman2_aux=sigman2_aux2;
                                                    break;
                                                end
                                                sigman2_aux=sigman2_aux2;
                                                if abs(sigman2_aux-sigman2_aux_prev)<err_est_sigman2_ML
                                                    conv_vec(2,MC_aux)=conv_ML_aux;
                                                    break;
                                                end
                                                
                                            end
                                            est_sigman2_unk_pilot_ML(m)=sigman2_aux;
                                            
                                            %%% ML Estimator (Alternative form). EM Newton - Raphson
                                            sigman2_aux=sigman2_initial_guess; % Initial guess
                                            s_vec=zeros(tao,M_cons^tao);
                                            T_est_vec=zeros(1,M_cons^tao);
                                            s_vec_aux=1;
                                            s_aux=zeros(tao,1);
                                            for s1=1:M_cons
                                                s_aux(1)=ideal_cons_points(s1);
                                                for s2=1:M_cons
                                                    s_aux(2)=ideal_cons_points(s2);
                                                    for s3=1:M_cons
                                                        s_aux(3)=ideal_cons_points(s3);
                                                        for s4=1:M_cons
                                                            s_aux(4)=ideal_cons_points(s4);
                                                            for s5=1:M_cons
                                                                s_aux(5)=ideal_cons_points(s5);
%                                                                 for s6=1:M_cons
%                                                                     s_aux(6)=ideal_cons_points(s6);
%                                                                     for s7=1:M_cons
%                                                                         s_aux(7)=ideal_cons_points(s7);
%                                                                         for s8=1:M_cons
%                                                                             s_aux(8)=ideal_cons_points(s8);
                                                                            s_vec(:,s_vec_aux)=s_aux;
                                                                            T_est_vec(s_vec_aux)=y_m'*(s_aux*s_aux')*y_m;
                                                                            s_vec_aux=s_vec_aux+1;
%                                                                         end
%                                                                     end
%                                                                 end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                            [~,max_i]=max(T_est_vec);
                                            s_max=s_vec(:,max_i);
                                            E_SS=s_max*s_max';
                                            
                                            while(1)
                                                %%% First derivative
                                                a_1st=(1/sigman2_aux^2)*(y_m'*y_m);
                                                b_1st=(((1/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-((1/tao)/(sigman2_aux^2)))*y_m'*(E_SS)*y_m;
                                                c_1st=(((tao*sum_R_coor(m,m,aux_L,K))+((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^2)-(tao/sigman2_aux);
                                                der_1st=(a_1st+b_1st+c_1st);
                                                
                                                %%% Second derivative
                                                a_2nd=-(2/sigman2_aux^3)*(y_m'*y_m);
                                                b_2nd=(((2/tao)/(sigman2_aux^3))-((2/tao)/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3))*y_m'*(E_SS)*y_m;
                                                c_2nd=-((((tao*sum_R_coor(m,m,aux_L,K))^3/(sigman2_aux^2))+3*((tao*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux))+(2*tao*sum_R_coor(m,m,aux_L,K)))/(sigman2_aux+tao*sum_R_coor(m,m,aux_L,K))^3)+(tao/(sigman2_aux^2));
                                                der_2nd=(a_2nd+b_2nd+c_2nd);
                                                
                                                est_sigman2_unk_pilot_ML_alt(m)=(sigman2_aux-der_1st/der_2nd);
                                                
                                                if abs(est_sigman2_unk_pilot_ML_alt(m)-sigman2_aux)<err_est_sigman2_ML
                                                    break;
                                                else
                                                    sigman2_aux=est_sigman2_unk_pilot_ML_alt(m);
                                                end
                                                
                                            end
                                            
                                            %%% CRLB (random and unknown channel)
                                            sum_T=0;
                                            sum_TD=0;
                                            sum_der=0;
                                            D_est_der_vec=zeros(1,M_cons^tao);
                                            T_est_vec=zeros(1,M_cons^tao);
                                            D_est_der_vec_aux=1;
                                            s_aux=zeros(tao,1);
                                            for s1=1:M_cons
                                                s_aux(1)=ideal_cons_points(s1);
                                                for s2=1:M_cons
                                                    s_aux(2)=ideal_cons_points(s2);
                                                    for s3=1:M_cons
                                                        s_aux(3)=ideal_cons_points(s3);
                                                        for s4=1:M_cons
                                                            s_aux(4)=ideal_cons_points(s4);
                                                            for s5=1:M_cons
                                                                s_aux(5)=ideal_cons_points(s5);
%                                                                 for s6=1:M_cons
%                                                                     s_aux(6)=ideal_cons_points(s6);
%                                                                     for s7=1:M_cons
%                                                                         s_aux(7)=ideal_cons_points(s7);
%                                                                         for s8=1:M_cons
%                                                                             s_aux(8)=ideal_cons_points(s8);
                                                                            T_est=exp((y_m'*(s_aux*s_aux')*y_m)/((sigman2(m)^2/sum_R_coor(m,m,aux_L,K))+sigman2(m)*tao));
                                                                            D_est=(((y_m'*(s_aux*s_aux')*y_m)/tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao).^2)-(y_m'*(s_aux*s_aux')*y_m)/(tao*sigman2(m)^2);
                                                                            D_est_der=((-2*(y_m'*(s_aux*s_aux')*y_m)/tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao).^3)+(2*y_m'*(s_aux*s_aux')*y_m)/(tao*sigman2(m)^3);
                                                                            sum_T=sum_T+T_est;
                                                                            sum_TD=sum_TD+T_est*D_est;
                                                                            sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                                            D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                                            T_est_vec(D_est_der_vec_aux)=y_m'*(s_aux*s_aux')*y_m;
                                                                            D_est_der_vec_aux=D_est_der_vec_aux+1;
%                                                                         end
%                                                                     end
%                                                                 end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                            der2=(sum_der/sum_T)-(((sum_TD)^2)/((sum_T)^2));
                                            [~,max_i]=max(T_est_vec);
                                            der2_alt=D_est_der_vec(max_i);
                                            crlb_unk_unk_pilot_res(m)=((tao)/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*tao)^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*tao)^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao)^3)-(2*(y_m'*y_m)/(sigman2(m)^3))+der2;
                                            crlb_unk_unk_pilot_res_alt(m)=((tao)/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*tao)^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*tao)^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao)^3)-(2*(y_m'*y_m)/(sigman2(m)^3))+der2_alt;
                                            
                                            %%% CRLB (random and known channel)
                                            if cmp_known_channel==1
                                                sum_R_coor_aux=sum_R_coor;
                                                sum_R_coor(:,:,aux_L,K)=zeros(M,M);
                                                sum_T=0;
                                                sum_TD=0;
                                                sum_der=0;
                                                D_est_der_vec=zeros(1,M_cons^tao);
                                                T_est_vec=zeros(1,M_cons^tao);
                                                D_est_der_vec_aux=1;
                                                s_aux=zeros(tao,1);
                                                for s1=1:M_cons
                                                    s_aux(1)=ideal_cons_points(s1);
                                                    for s2=1:M_cons
                                                        s_aux(2)=ideal_cons_points(s2);
                                                        for s3=1:M_cons
                                                            s_aux(3)=ideal_cons_points(s3);
                                                            for s4=1:M_cons
                                                                s_aux(4)=ideal_cons_points(s4);
                                                                for s5=1:M_cons
                                                                    s_aux(5)=ideal_cons_points(s5);
%                                                                     for s6=1:M_cons
%                                                                         s_aux(6)=ideal_cons_points(s6);
%                                                                         for s7=1:M_cons
%                                                                             s_aux(7)=ideal_cons_points(s7);
%                                                                             for s8=1:M_cons
%                                                                                 s_aux(8)=ideal_cons_points(s8);
                                                                                T_est=exp((y_m'*(s_aux*s_aux')*y_m)/((sigman2(m)^2/sum_R_coor(m,m,aux_L,K))+sigman2(m)*tao));
                                                                                D_est=(((y_m'*(s_aux*s_aux')*y_m)/tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao).^2)-(y_m'*(s_aux*s_aux')*y_m)/(tao*sigman2(m)^2);
                                                                                D_est_der=((-2*(y_m'*(s_aux*s_aux')*y_m)/tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao).^3)+(2*y_m'*(s_aux*s_aux')*y_m)/(tao*sigman2(m)^3);
                                                                                sum_T=sum_T+T_est;
                                                                                sum_TD=sum_TD+T_est*D_est;
                                                                                sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                                                D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                                                T_est_vec(D_est_der_vec_aux)=y_m'*(s_aux*s_aux')*y_m;
                                                                                D_est_der_vec_aux=D_est_der_vec_aux+1;
%                                                                             end
%                                                                         end
%                                                                     end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                                der2=(sum_der/sum_T)-(((sum_TD)^2)/((sum_T)^2));
                                                [~,max_i]=max(T_est_vec);
                                                der2_alt=D_est_der_vec(max_i);
                                                crlb_k_unk_pilot_res(m)=((tao)/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*tao)^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*tao)^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao)^3)-(2*(y_m'*y_m)/(sigman2(m)^3))+der2;
                                                crlb_k_unk_pilot_res_alt(m)=((tao)/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*tao)^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*tao)^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*tao)./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*tao)^3)-(2*(y_m'*y_m)/(sigman2(m)^3))+der2_alt;
                                                sum_R_coor=sum_R_coor_aux;
                                            end
                                            
                                            %% Mixed pilot
                                            %%% MM Estimator
                                            est_sigman2_mix_pilot_MM(m)=((y_mix_m'*y_mix_m/(tao+kappa)))-sum_R_coor(m,m,aux_L,K);
                                            
                                            %%% ML Estimator. EM Newton - Raphson
                                            sigman2_aux=sigman2_initial_guess; % Initial guess
                                            conv_ML_aux=0;
                                            while(1)
                                                conv_ML_aux=conv_ML_aux+1;
                                                sigman2_aux_prev=sigman2_aux;
                                                E_SS=0;
                                                sum_pdf_yS=0;
                                                s_aux=zeros((tao+kappa),1);
                                                s_aux(1:tao)=s(1:tao,aux_L);
                                                for sunk_1=1:M_cons
                                                    s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                    for sunk_2=1:M_cons
                                                        s_aux(tao+2)=ideal_cons_points(sunk_2);
%                                                         for sunk_3=1:M_cons
%                                                             s_aux(tao+3)=ideal_cons_points(sunk_3);
%                                                             for sunk_4=1:M_cons
%                                                                 s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                                %                                                                 for sunk_5=1:M_cons
                                                                %                                                                     s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                                %                                                                     for sunk_6=1:M_cons
                                                                %                                                                         s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                                %                                                                         for sunk_7=1:M_cons
                                                                %                                                                             s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                                %                                                                             for sunk_8=1:M_cons
                                                                %                                                                                 s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                                det_Cy=(sigman2_aux)^(tao+kappa)+(tao+kappa)*sum_R_coor(m,m,aux_L,K)*(sigman2_aux)^((tao+kappa)-1);
                                                                inv_Cy=(1/sigman2_aux)*((eye((tao+kappa),(tao+kappa)))-(s_aux*s_aux')/((sigman2_aux/sum_R_coor(m,m,aux_L,K))+(tao+kappa)));
                                                                pdf_y_S=(1/((pi^(tao+kappa))*det_Cy))*exp(-y_mix_m'*inv_Cy*y_mix_m);
                                                                E_SS=E_SS+s_aux*s_aux'*pdf_y_S;
                                                                sum_pdf_yS=sum_pdf_yS+pdf_y_S;
                                                                %                                                                             end
                                                                %                                                                         end
                                                                %                                                                     end
                                                                %                                                                 end
%                                                             end
%                                                         end
                                                    end
                                                end
                                                E_SS=E_SS/sum_pdf_yS;
                                                
                                                while(1)
                                                    %%% First derivative
                                                    a_1st=(1/sigman2_aux^2)*(y_mix_m'*y_mix_m);
                                                    b_1st=(((1/(tao+kappa))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^2)-((1/(tao+kappa))/(sigman2_aux^2)))*y_mix_m'*(E_SS)*y_mix_m;
                                                    c_1st=((((tao+kappa)*sum_R_coor(m,m,aux_L,K))+(((tao+kappa)*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux)))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^2)-((tao+kappa)/sigman2_aux);
                                                    der_1st=(a_1st+b_1st+c_1st);
                                                    
                                                    %%% Second derivative
                                                    a_2nd=-(2/sigman2_aux^3)*(y_mix_m'*y_mix_m);
                                                    b_2nd=(((2/(tao+kappa))/(sigman2_aux^3))-((2/(tao+kappa))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^3))*y_mix_m'*(E_SS)*y_mix_m;
                                                    c_2nd=-(((((tao+kappa)*sum_R_coor(m,m,aux_L,K))^3/(sigman2_aux^2))+3*(((tao+kappa)*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux))+(2*(tao+kappa)*sum_R_coor(m,m,aux_L,K)))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^3)+((tao+kappa)/(sigman2_aux^2));
                                                    der_2nd=(a_2nd+b_2nd+c_2nd);
                                                    
                                                    sigman2_aux2=(sigman2_aux-der_1st/der_2nd);
                                                    if isnan(sigman2_aux2)
                                                        break;
                                                    end
                                                    
                                                    if abs(sigman2_aux2-sigman2_aux)<err_est_sigman2_ML
                                                        break;
                                                    else
                                                        sigman2_aux=sigman2_aux2;
                                                    end
                                                    
                                                end
                                                if isnan(sigman2_aux2)
                                                    sigman2_aux=sigman2_aux2;
                                                    break;
                                                end
                                                sigman2_aux=sigman2_aux2;
                                                if abs(sigman2_aux-sigman2_aux_prev)<err_est_sigman2_ML
                                                    conv_vec(2,MC_aux)=conv_ML_aux;
                                                    break;
                                                end
                                                
                                            end
                                            est_sigman2_mix_pilot_ML(m)=sigman2_aux;
                                            
                                            %%% ML Estimator (Alternative form). EM Newton - Raphson
                                            sigman2_aux=sigman2_initial_guess; % Initial guess
                                            s_vec=zeros((tao+kappa),M_cons^kappa);
                                            T_est_vec=zeros(1,M_cons^kappa);
                                            s_vec_aux=1;
                                            s_aux=zeros((tao+kappa),1);
                                            s_aux(1:tao)=s(1:tao,aux_L);
                                            for sunk_1=1:M_cons
                                                s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                for sunk_2=1:M_cons
                                                    s_aux(tao+2)=ideal_cons_points(sunk_2);
%                                                     for sunk_3=1:M_cons
%                                                         s_aux(tao+3)=ideal_cons_points(sunk_3);
%                                                         for sunk_4=1:M_cons
%                                                             s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                            %                                                                 for sunk_5=1:M_cons
                                                            %                                                                     s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                            %                                                                     for sunk_6=1:M_cons
                                                            %                                                                         s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                            %                                                                         for sunk_7=1:M_cons
                                                            %                                                                             s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                            %                                                                             for sunk_8=1:M_cons
                                                            %                                                                                 s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                            s_vec(:,s_vec_aux)=s_aux;
                                                            T_est_vec(s_vec_aux)=y_mix_m'*(s_aux*s_aux')*y_mix_m;
                                                            s_vec_aux=s_vec_aux+1;
                                                            %                                                                             end
                                                            %                                                                         end
                                                            %                                                                     end
                                                            %                                                                 end
%                                                         end
%                                                     end
                                                end
                                            end
                                            [~,max_i]=max(T_est_vec);
                                            s_max=s_vec(:,max_i);
                                            E_SS=s_max*s_max';
                                            
                                            while(1)
                                                %%% First derivative
                                                a_1st=(1/sigman2_aux^2)*(y_mix_m'*y_mix_m);
                                                b_1st=(((1/(tao+kappa))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^2)-((1/(tao+kappa))/(sigman2_aux^2)))*y_mix_m'*(E_SS)*y_mix_m;
                                                c_1st=((((tao+kappa)*sum_R_coor(m,m,aux_L,K))+(((tao+kappa)*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux)))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^2)-((tao+kappa)/sigman2_aux);
                                                der_1st=(a_1st+b_1st+c_1st);
                                                
                                                %%% Second derivative
                                                a_2nd=-(2/sigman2_aux^3)*(y_mix_m'*y_mix_m);
                                                b_2nd=(((2/(tao+kappa))/(sigman2_aux^3))-((2/(tao+kappa))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^3))*y_mix_m'*(E_SS)*y_mix_m;
                                                c_2nd=-(((((tao+kappa)*sum_R_coor(m,m,aux_L,K))^3/(sigman2_aux^2))+3*(((tao+kappa)*sum_R_coor(m,m,aux_L,K))^2/(sigman2_aux))+(2*(tao+kappa)*sum_R_coor(m,m,aux_L,K)))/(sigman2_aux+(tao+kappa)*sum_R_coor(m,m,aux_L,K))^3)+((tao+kappa)/(sigman2_aux^2));
                                                der_2nd=(a_2nd+b_2nd+c_2nd);
                                                
                                                est_sigman2_mix_pilot_ML_alt(m)=(sigman2_aux-der_1st/der_2nd);
                                                
                                                if abs(est_sigman2_mix_pilot_ML_alt(m)-sigman2_aux)<err_est_sigman2_ML
                                                    break;
                                                else
                                                    sigman2_aux=est_sigman2_mix_pilot_ML_alt(m);
                                                end
                                                
                                            end
                                            
                                            %%% CRLB (random and unknown channel)
                                            sum_T=0;
                                            sum_TD=0;
                                            sum_der=0;
                                            D_est_der_vec=zeros(1,M_cons^kappa);
                                            T_est_vec=zeros(1,M_cons^kappa);
                                            D_est_der_vec_aux=1;
                                            s_aux=zeros((tao+kappa),1);
                                            s_aux(1:tao)=s(1:tao,aux_L);
                                            for sunk_1=1:M_cons
                                                s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                for sunk_2=1:M_cons
                                                    s_aux(tao+2)=ideal_cons_points(sunk_2);
%                                                     for sunk_3=1:M_cons
%                                                         s_aux(tao+3)=ideal_cons_points(sunk_3);
%                                                         for sunk_4=1:M_cons
%                                                             s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                            %                                                             for sunk_5=1:M_cons
                                                            %                                                                 s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                            %                                                                 for sunk_6=1:M_cons
                                                            %                                                                     s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                            %                                                                     for sunk_7=1:M_cons
                                                            %                                                                         s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                            %                                                                         for sunk_8=1:M_cons
                                                            %                                                                             s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                            
                                                            T_est=exp((y_mix_m'*(s_aux*s_aux')*y_mix_m)/((sigman2(m)^2/sum_R_coor(m,m,aux_L,K))+sigman2(m)*(tao+kappa)));
                                                            D_est=(((y_mix_m'*(s_aux*s_aux')*y_mix_m)/(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa)).^2)-(y_mix_m'*(s_aux*s_aux')*y_mix_m)/((tao+kappa)*sigman2(m)^2);
                                                            D_est_der=((-2*(y_mix_m'*(s_aux*s_aux')*y_mix_m)/(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa)).^3)+(2*y_mix_m'*(s_aux*s_aux')*y_mix_m)/((tao+kappa)*sigman2(m)^3);
                                                            sum_T=sum_T+T_est;
                                                            sum_TD=sum_TD+T_est*D_est;
                                                            sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                            D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                            T_est_vec(D_est_der_vec_aux)=y_mix_m'*(s_aux*s_aux')*y_mix_m;
                                                            D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                            %                                                                     end
                                                            %                                                                 end
                                                            %                                                             end
                                                            %             end
%                                                         end
%                                                     end
                                                end
                                            end
                                            der2=(sum_der/sum_T)-(((sum_TD)^2)/((sum_T)^2));
                                            [~,max_i]=max(T_est_vec);
                                            der2_alt=D_est_der_vec(max_i);
                                            crlb_unk_mix_pilot_res(m)=(((tao+kappa))/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*(tao+kappa))^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3)-(2*(y_mix_m'*y_mix_m)/(sigman2(m)^3))+der2;
                                            crlb_unk_mix_pilot_res_alt(m)=(((tao+kappa))/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*(tao+kappa))^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3)-(2*(y_mix_m'*y_mix_m)/(sigman2(m)^3))+der2_alt;
                                            
                                            %%% CRLB (random and known channel)
                                            if cmp_known_channel==1
                                                sum_R_coor_aux=sum_R_coor;
                                                sum_R_coor(:,:,aux_L,K)=zeros(M,M);
                                                sum_T=0;
                                                sum_TD=0;
                                                sum_der=0;
                                                D_est_der_vec=zeros(1,M_cons^kappa);
                                                T_est_vec=zeros(1,M_cons^kappa);
                                                D_est_der_vec_aux=1;
                                                s_aux=zeros((tao+kappa),1);
                                                s_aux(1:tao)=s(1:tao,aux_L);
                                                for sunk_1=1:M_cons
                                                    s_aux(tao+1)=ideal_cons_points(sunk_1);
                                                    for sunk_2=1:M_cons
                                                        s_aux(tao+2)=ideal_cons_points(sunk_2);
                                                        %                                                         for sunk_3=1:M_cons
                                                        %                                                             s_aux(tao+3)=ideal_cons_points(sunk_3);
                                                        %                                                             for sunk_4=1:M_cons
                                                        %                                                                 s_aux(tao+4)=ideal_cons_points(sunk_4);
                                                        %                                                             for sunk_5=1:M_cons
                                                        %                                                                 s_aux(tao+5)=ideal_cons_points(sunk_5);
                                                        %                                                                 for sunk_6=1:M_cons
                                                        %                                                                     s_aux(tao+6)=ideal_cons_points(sunk_6);
                                                        %                                                                     for sunk_7=1:M_cons
                                                        %                                                                         s_aux(tao+7)=ideal_cons_points(sunk_7);
                                                        %                                                                         for sunk_8=1:M_cons
                                                        %                                                                             s_aux(tao+8)=ideal_cons_points(sunk_8);
                                                        
                                                        T_est=exp((y_mix_m'*(s_aux*s_aux')*y_mix_m)/((sigman2(m)^2/sum_R_coor(m,m,aux_L,K))+sigman2(m)*(tao+kappa)));
                                                        D_est=(((y_mix_m'*(s_aux*s_aux')*y_mix_m)/(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa)).^2)-(y_mix_m'*(s_aux*s_aux')*y_mix_m)/((tao+kappa)*sigman2(m)^2);
                                                        D_est_der=((-2*(y_mix_m'*(s_aux*s_aux')*y_mix_m)/(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa)).^3)+(2*y_mix_m'*(s_aux*s_aux')*y_mix_m)/((tao+kappa)*sigman2(m)^3);
                                                        sum_T=sum_T+T_est;
                                                        sum_TD=sum_TD+T_est*D_est;
                                                        sum_der=sum_der+((T_est*(D_est^2))+T_est*D_est_der);
                                                        D_est_der_vec(D_est_der_vec_aux)=D_est_der;
                                                        T_est_vec(D_est_der_vec_aux)=y_mix_m'*(s_aux*s_aux')*y_mix_m;
                                                        D_est_der_vec_aux=D_est_der_vec_aux+1;
                                                        %                                                                     end
                                                        %                                                                 end
                                                        %                                                             end
                                                        %             end
                                                        %                                                             end
                                                        %                                                         end
                                                    end
                                                end
                                                der2=(sum_der/sum_T)-(((sum_TD)^2)/((sum_T)^2));
                                                [~,max_i]=max(T_est_vec);
                                                der2_alt=D_est_der_vec(max_i);
                                                crlb_k_mix_pilot_res(m)=(((tao+kappa))/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*(tao+kappa))^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3)-(2*(y_mix_m'*y_mix_m)/(sigman2(m)^3))+der2;
                                                crlb_k_mix_pilot_res_alt(m)=(((tao+kappa))/(sigman2(m)^2))-((((sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3/(sigman2(m)^2))+(3*(sum_R_coor(m,m,aux_L,K)*(tao+kappa))^2/sigman2(m))+2*sum_R_coor(m,m,aux_L,K)*(tao+kappa))./(sigman2(m)+sum_R_coor(m,m,aux_L,K)*(tao+kappa))^3)-(2*(y_mix_m'*y_mix_m)/(sigman2(m)^3))+der2_alt;
                                                sum_R_coor=sum_R_coor_aux;
                                            end
                                            
                                            %
                                            %                                     noise_est_vec_ML(noise_est_vec_ML_aux)=est_sigman2_ML;
                                            %                                     noise_est_vec_MM(noise_est_vec_MM_aux)=est_sigman2_MM;
                                            %                                     noise_est_vec_MM_aux=noise_est_vec_MM_aux+1;
                                            %                                     noise_est_vec_ML_aux=noise_est_vec_ML_aux+1;                                        end
                                            
                                        end
                                        
                                    end
                                    
                                end
                                
                                %% CPA Estimator (with noise variance estimation (ML) )
                                h1_est_CPA_noise_ML=R_coor(:,:,aux_L,aux_L,K)/(diag(est_sigman2_mix_pilot_ML_alt)+tao*sum_R_coor(:,:,aux_L,K))*S'*y_coor;
                                
                                %% CPA Estimator (with noise variance estimation (MM) )
                                h1_est_CPA_noise_MM=R_coor(:,:,aux_L,aux_L,K)/(diag(est_sigman2_mix_pilot_MM)+tao*sum_R_coor(:,:,aux_L,K))*S'*y_coor;
                                
                                %% CPA Estimator (with noise variance uncertainty (1 dB) )
                                h1_est_CPA_noise_NU_1dB=R_coor(:,:,aux_L,aux_L,K)/(diag(sigman2*(10^(3/10)))+tao*sum_R_coor(:,:,aux_L,K))*S'*y_coor;
                                
                                if comp_cap==1
                                    H_est(:,K,aux_L)=h1_est_CB;
                                    for l=1:L
                                        %H(:,aux_L,l,K)=h_coor(:,l);
                                        H(:,aux_L,l,K)=h(:,l);
                                    end
                                                                        
                                else
                                    break;
                                end
                                
                            end
                            
                            %% Results (1)
                            if error_vs_ant_num==1 || rate_vs_ant_num==1
                                
                                S_sim(aux_M)=S_sim(aux_M)+sum(sum(((abs(Y_hs).^2))));
                                N_sim(aux_M)=N_sim(aux_M)+sum(sum(((abs(N).^2))));
                                
                                if error_vs_ant_num==1
                                    
                                    %%%% Estimation Error, Number of antennas %%%%
                                    err_LS_num(aux_M)=err_LS_num(aux_M)+((norm(h1_est_LS-h(:,aux_L),'fro')^2));
                                    err_LS_no_int_num(aux_M)=err_LS_no_int_num(aux_M)+((norm(h1_est_LS_no_int-h(:,aux_L),'fro')^2));
                                    err_CB_num(aux_M)=err_CB_num(aux_M)+((norm(h1_est_CB-h(:,aux_L),'fro')^2));
                                    err_CB_no_int_num(aux_M)=err_CB_no_int_num(aux_M)+((norm(h1_est_CB_no_int-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_num(aux_M)=err_CPA_num(aux_M)+((norm(h1_est_CPA-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_ML_num(aux_M)=err_CPA_noise_ML_num(aux_M)+((norm(h1_est_CPA_noise_ML-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_MM_num(aux_M)=err_CPA_noise_MM_num(aux_M)+((norm(h1_est_CPA_noise_MM-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_NU_1dB_num(aux_M)=err_CPA_noise_NU_1dB_num(aux_M)+((norm(h1_est_CPA_noise_NU_1dB-h_coor(:,aux_L),'fro')^2));
                                    
                                    err_sigman2_ML_num(aux_M)=err_sigman2_ML_num(aux_M)+(norm(est_sigman2_ML-sigman2,'fro')^2);
                                    err_sigman2_MM_num(aux_M)=err_sigman2_MM_num(aux_M)+(norm(est_sigman2_MM-sigman2,'fro')^2);
                                    err_sigman2_LS_num(aux_M)=err_sigman2_LS_num(aux_M)+(norm(est_sigman2_LS-sigman2,'fro')^2);
                                    err_sigman2_LS_PSD_num(aux_M)=err_sigman2_LS_PSD_num(aux_M)+(norm(est_sigman2_LS_PSD-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_num(aux_M)=err_sigman2_unk_pilot_ML_num(aux_M)+(norm(est_sigman2_unk_pilot_ML-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_num_alt(aux_M)=err_sigman2_unk_pilot_ML_num_alt(aux_M)+(norm(est_sigman2_unk_pilot_ML_alt-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_MM_num(aux_M)=err_sigman2_unk_pilot_MM_num(aux_M)+(norm(est_sigman2_unk_pilot_MM-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_num(aux_M)=err_sigman2_mix_pilot_ML_num(aux_M)+(norm(est_sigman2_mix_pilot_ML-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_num_alt(aux_M)=err_sigman2_mix_pilot_ML_num_alt(aux_M)+(norm(est_sigman2_mix_pilot_ML_alt-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_MM_num(aux_M)=err_sigman2_mix_pilot_MM_num(aux_M)+(norm(est_sigman2_mix_pilot_MM-sigman2,'fro')^2);
                                    crlb_unk_num(aux_M)=crlb_unk_num(aux_M)+sum(crlb_unk_res);
                                    crlb_k_num(aux_M)=crlb_k_num(aux_M)+sum(crlb_k_res);
                                    crlb_unk_pilot_num(1:M,aux_M)=crlb_unk_pilot_num(1:M,aux_M)+crlb_unk_unk_pilot_res.';
                                    crlb_k_unk_pilot_num(1:M,aux_M)=crlb_k_unk_pilot_num(1:M,aux_M)+crlb_k_unk_pilot_res.';
                                    crlb_unk_pilot_num_alt(1:M,aux_M)=crlb_unk_pilot_num_alt(1:M,aux_M)+crlb_unk_unk_pilot_res_alt.';
                                    crlb_k_unk_pilot_num_alt(1:M,aux_M)=crlb_k_unk_pilot_num_alt(1:M,aux_M)+crlb_k_unk_pilot_res_alt.';
                                    crlb_mix_pilot_num(1:M,aux_M)=crlb_mix_pilot_num(1:M,aux_M)+crlb_unk_mix_pilot_res.';
                                    crlb_k_mix_pilot_num(1:M,aux_M)=crlb_k_mix_pilot_num(1:M,aux_M)+crlb_k_mix_pilot_res.';
                                    crlb_mix_pilot_num_alt(1:M,aux_M)=crlb_mix_pilot_num_alt(1:M,aux_M)+crlb_unk_mix_pilot_res_alt.';
                                    crlb_k_mix_pilot_num_alt(1:M,aux_M)=crlb_k_mix_pilot_num_alt(1:M,aux_M)+crlb_k_mix_pilot_res_alt.';
                                    err_LS_den(aux_M)=err_LS_den(aux_M)+((norm(h(:,aux_L),'fro')^2));
                                    err_LS_no_int_den(aux_M)=err_LS_no_int_den(aux_M)+((norm(h(:,aux_L),'fro')^2));
                                    err_CB_den(aux_M)=err_CB_den(aux_M)+((norm(h(:,aux_L),'fro')^2));
                                    err_CB_no_int_den(aux_M)=err_CB_no_int_den(aux_M)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_den(aux_M)=err_CPA_den(aux_M)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_ML_den(aux_M)=err_CPA_noise_ML_den(aux_M)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_MM_den(aux_M)=err_CPA_noise_MM_den(aux_M)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_NU_1dB_den(aux_M)=err_CPA_noise_NU_1dB_den(aux_M)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_sigman2_ML_den(aux_M)=err_sigman2_ML_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_MM_den(aux_M)=err_sigman2_MM_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_LS_den(aux_M)=err_sigman2_LS_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_LS_PSD_den(aux_M)=err_sigman2_LS_PSD_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_den(aux_M)=err_sigman2_unk_pilot_ML_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_den_alt(aux_M)=err_sigman2_unk_pilot_ML_den_alt(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_MM_den(aux_M)=err_sigman2_unk_pilot_MM_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_den(aux_M)=err_sigman2_mix_pilot_ML_den(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_den_alt(aux_M)=err_sigman2_mix_pilot_ML_den_alt(aux_M)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_MM_den(aux_M)=err_sigman2_mix_pilot_MM_den(aux_M)+(norm(sigman2,'fro')^2);
                                    crlb_unk_den(aux_M)=crlb_unk_den(aux_M)+(norm(sigman2,'fro')^2);
                                    crlb_k_den(aux_M)=crlb_k_den(aux_M)+(norm(sigman2,'fro')^2);
                                    crlb_unk_pilot_den(1:M,aux_M)=crlb_unk_pilot_den(1:M,aux_M)+1;
                                    crlb_k_unk_pilot_den(1:M,aux_M)=crlb_k_unk_pilot_den(1:M,aux_M)+1;
                                    crlb_unk_pilot_den_alt(1:M,aux_M)=crlb_unk_pilot_den_alt(1:M,aux_M)+1;
                                    crlb_k_unk_pilot_den_alt(1:M,aux_M)=crlb_k_unk_pilot_den_alt(1:M,aux_M)+1;
                                    crlb_mix_pilot_den(1:M,aux_M)=crlb_mix_pilot_den(1:M,aux_M)+1;
                                    crlb_k_mix_pilot_den(1:M,aux_M)=crlb_k_mix_pilot_den(1:M,aux_M)+1;
                                    crlb_mix_pilot_den_alt(1:M,aux_M)=crlb_mix_pilot_den_alt(1:M,aux_M)+1;
                                    crlb_k_mix_pilot_den_alt(1:M,aux_M)=crlb_k_mix_pilot_den_alt(1:M,aux_M)+1;
                                end
                                
                                if rate_vs_ant_num==1
                                    
                                    %%%% Per-cell Rate, Number of antennas %%%%
                                    
                                end
                            end
                            
                            if error_vs_std_dev_AOA==1
                                
                                S_sim(aux_sigma_AOA)=S_sim(aux_sigma_AOA)+sum(sum(((abs(Y_hs).^2))));
                                N_sim(aux_sigma_AOA)=N_sim(aux_sigma_AOA)+sum(sum(((abs(N).^2))));
                                
                                if error_vs_std_dev_AOA==1
                                    
                                    %%%% Estimation Error, Standard Deviation of AOA %%%%
                                    err_LS_num(aux_sigma_AOA)=err_LS_num(aux_sigma_AOA)+((norm(h1_est_LS-h(:,aux_L),'fro')^2));
                                    err_LS_no_int_num(aux_sigma_AOA)=err_LS_no_int_num(aux_sigma_AOA)+((norm(h1_est_LS_no_int-h(:,aux_L),'fro')^2));
                                    err_CB_num(aux_sigma_AOA)=err_CB_num(aux_sigma_AOA)+((norm(h1_est_CB-h(:,aux_L),'fro')^2));
                                    err_CB_no_int_num(aux_sigma_AOA)=err_CB_no_int_num(aux_sigma_AOA)+((norm(h1_est_CB_no_int-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_num(aux_sigma_AOA)=err_CPA_num(aux_sigma_AOA)+((norm(h1_est_CPA-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_ML_num(aux_sigma_AOA)=err_CPA_noise_ML_num(aux_sigma_AOA)+((norm(h1_est_CPA_noise_ML-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_MM_num(aux_sigma_AOA)=err_CPA_noise_MM_num(aux_sigma_AOA)+((norm(h1_est_CPA_noise_MM-h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_NU_1dB_num(aux_sigma_AOA)=err_CPA_noise_NU_1dB_num(aux_sigma_AOA)+((norm(h1_est_CPA_noise_NU_1dB-h_coor(:,aux_L),'fro')^2));
                                    err_sigman2_ML_num(aux_sigma_AOA)=err_sigman2_ML_num(aux_sigma_AOA)+(norm(est_sigman2_ML-sigman2,'fro')^2);
                                    err_sigman2_MM_num(aux_sigma_AOA)=err_sigman2_MM_num(aux_sigma_AOA)+(norm(est_sigman2_MM-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_num(aux_sigma_AOA)=err_sigman2_unk_pilot_ML_num(aux_sigma_AOA)+(norm(est_sigman2_unk_pilot_ML-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_num_alt(aux_sigma_AOA)=err_sigman2_unk_pilot_ML_num_alt(aux_sigma_AOA)+(norm(est_sigman2_unk_pilot_ML_alt-sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_MM_num(aux_sigma_AOA)=err_sigman2_unk_pilot_MM_num(aux_sigma_AOA)+(norm(est_sigman2_unk_pilot_MM-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_num(aux_sigma_AOA)=err_sigman2_mix_pilot_ML_num(aux_sigma_AOA)+(norm(est_sigman2_mix_pilot_ML-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_num_alt(aux_sigma_AOA)=err_sigman2_mix_pilot_ML_num_alt(aux_sigma_AOA)+(norm(est_sigman2_mix_pilot_ML_alt-sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_MM_num(aux_sigma_AOA)=err_sigman2_mix_pilot_MM_num(aux_sigma_AOA)+(norm(est_sigman2_mix_pilot_MM-sigman2,'fro')^2);
                                    crlb_unk_num(aux_sigma_AOA)=crlb_unk_num(aux_sigma_AOA)+sum(crlb_unk_res);
                                    crlb_k_num(aux_sigma_AOA)=crlb_k_num(aux_sigma_AOA)+sum(crlb_k_res);
                                    crlb_unk_pilot_num(1:M,aux_sigma_AOA)=crlb_unk_pilot_num(1:M,aux_sigma_AOA)+crlb_unk_unk_pilot_res.';
                                    crlb_k_unk_pilot_num(1:M,aux_sigma_AOA)=crlb_k_unk_pilot_num(1:M,aux_sigma_AOA)+crlb_k_unk_pilot_res.';
                                    crlb_unk_pilot_num_alt(1:M,aux_sigma_AOA)=crlb_unk_pilot_num_alt(1:M,aux_sigma_AOA)+crlb_unk_unk_pilot_res_alt.';
                                    crlb_k_unk_pilot_num_alt(1:M,aux_sigma_AOA)=crlb_k_unk_pilot_num_alt(1:M,aux_sigma_AOA)+crlb_k_unk_pilot_res_alt.';
                                    crlb_mix_pilot_num(1:M,aux_sigma_AOA)=crlb_mix_pilot_num(1:M,aux_sigma_AOA)+crlb_unk_mix_pilot_res.';
                                    crlb_k_mix_pilot_num(1:M,aux_sigma_AOA)=crlb_k_mix_pilot_num(1:M,aux_sigma_AOA)+crlb_k_mix_pilot_res.';
                                    crlb_mix_pilot_num_alt(1:M,aux_sigma_AOA)=crlb_mix_pilot_num_alt(1:M,aux_sigma_AOA)+crlb_unk_mix_pilot_res_alt.';
                                    crlb_k_mix_pilot_num_alt(1:M,aux_sigma_AOA)=crlb_k_mix_pilot_num_alt(1:M,aux_sigma_AOA)+crlb_k_mix_pilot_res_alt.';
                                    err_LS_den(aux_sigma_AOA)=err_LS_den(aux_sigma_AOA)+((norm(h(:,aux_L),'fro')^2));
                                    err_LS_no_int_den(aux_sigma_AOA)=err_LS_no_int_den(aux_sigma_AOA)+((norm(h(:,aux_L),'fro')^2));
                                    err_CB_den(aux_sigma_AOA)=err_CB_den(aux_sigma_AOA)+((norm(h(:,aux_L),'fro')^2));
                                    err_CB_no_int_den(aux_sigma_AOA)=err_CB_no_int_den(aux_sigma_AOA)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_den(aux_sigma_AOA)=err_CPA_den(aux_sigma_AOA)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_ML_den(aux_sigma_AOA)=err_CPA_noise_ML_den(aux_sigma_AOA)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_MM_den(aux_sigma_AOA)=err_CPA_noise_MM_den(aux_sigma_AOA)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_CPA_noise_NU_1dB_den(aux_sigma_AOA)=err_CPA_noise_NU_1dB_den(aux_sigma_AOA)+((norm(h_coor(:,aux_L),'fro')^2));
                                    err_sigman2_ML_den(aux_sigma_AOA)=err_sigman2_ML_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_MM_den(aux_sigma_AOA)=err_sigman2_MM_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_den(aux_sigma_AOA)=err_sigman2_unk_pilot_ML_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_ML_den_alt(aux_sigma_AOA)=err_sigman2_unk_pilot_ML_den_alt(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_unk_pilot_MM_den(aux_sigma_AOA)=err_sigman2_unk_pilot_MM_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_den(aux_sigma_AOA)=err_sigman2_mix_pilot_ML_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_ML_den_alt(aux_sigma_AOA)=err_sigman2_mix_pilot_ML_den_alt(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    err_sigman2_mix_pilot_MM_den(aux_sigma_AOA)=err_sigman2_mix_pilot_MM_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    crlb_unk_den(aux_sigma_AOA)=crlb_unk_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    crlb_k_den(aux_sigma_AOA)=crlb_k_den(aux_sigma_AOA)+(norm(sigman2,'fro')^2);
                                    crlb_unk_pilot_den(1:M,aux_sigma_AOA)=crlb_unk_pilot_den(1:M,aux_sigma_AOA)+1;
                                    crlb_k_unk_pilot_den(1:M,aux_sigma_AOA)=crlb_k_unk_pilot_den(1:M,aux_sigma_AOA)+1;
                                    crlb_unk_pilot_den_alt(1:M,aux_sigma_AOA)=crlb_unk_pilot_den_alt(1:M,aux_sigma_AOA)+1;
                                    crlb_k_unk_pilot_den_alt(1:M,aux_sigma_AOA)=crlb_k_unk_pilot_den_alt(1:M,aux_sigma_AOA)+1;
                                    crlb_mix_pilot_den(1:M,aux_sigma_AOA)=crlb_mix_pilot_den(1:M,aux_sigma_AOA)+1;
                                    crlb_k_mix_pilot_den(1:M,aux_sigma_AOA)=crlb_k_mix_pilot_den(1:M,aux_sigma_AOA)+1;
                                    crlb_mix_pilot_den_alt(1:M,aux_sigma_AOA)=crlb_mix_pilot_den_alt(1:M,aux_sigma_AOA)+1;
                                    crlb_k_mix_pilot_den_alt(1:M,aux_sigma_AOA)=crlb_k_mix_pilot_den_alt(1:M,aux_sigma_AOA)+1;
                                end
                                
                                if rate_vs_std_dev_AOA==1
                                    
                                    %%%% Per-cell Rate, Standard Deviation of AOA %%%%
                                    
                                end
                            end
                            
                        end
                        
                        if error_vs_ant_num==1 || rate_vs_ant_num==1
                            
                            if error_vs_ant_num==1
                                
                                %%%% Estimation Error, Number of antennas %%%%
                                err_LS(aux_M)=err_LS(aux_M)+(err_LS_num(aux_M)/err_LS_den(aux_M));
                                err_LS_no_int(aux_M)=err_LS_no_int(aux_M)+(err_LS_no_int_num(aux_M)/err_LS_no_int_den(aux_M));
                                err_CB(aux_M)=err_CB(aux_M)+(err_CB_num(aux_M)/err_CB_den(aux_M));
                                err_CB_no_int(aux_M)=err_CB_no_int(aux_M)+(err_CB_no_int_num(aux_M)/err_CB_no_int_den(aux_M));
                                
                                err_CPA(aux_M)=err_CPA(aux_M)+10*log10(err_CPA_num(aux_M)/err_CPA_den(aux_M));
                                err_CPA_noise_ML(aux_M)=err_CPA_noise_ML(aux_M)+10*log10(err_CPA_noise_ML_num(aux_M)/err_CPA_noise_ML_den(aux_M));
                                err_CPA_noise_MM(aux_M)=err_CPA_noise_MM(aux_M)+10*log10(err_CPA_noise_MM_num(aux_M)/err_CPA_noise_MM_den(aux_M));
                                err_CPA_noise_NU_1dB(aux_M)=err_CPA_noise_NU_1dB(aux_M)+10*log10(err_CPA_noise_NU_1dB_num(aux_M)/err_CPA_noise_NU_1dB_den(aux_M));
                                
                                err_sigman2_ML(aux_M)=err_sigman2_ML(aux_M)+(err_sigman2_ML_num(aux_M)/err_sigman2_ML_den(aux_M));
                                err_sigman2_MM(aux_M)=err_sigman2_MM(aux_M)+(err_sigman2_MM_num(aux_M)/err_sigman2_MM_den(aux_M));
                                err_sigman2_LS(aux_M)=err_sigman2_LS(aux_M)+(err_sigman2_LS_num(aux_M)/err_sigman2_LS_den(aux_M));
                                err_sigman2_LS_PSD(aux_M)=err_sigman2_LS_PSD(aux_M)+(err_sigman2_LS_PSD_num(aux_M)/err_sigman2_LS_PSD_den(aux_M));
                                err_sigman2_unk_pilot_ML(aux_M)=err_sigman2_unk_pilot_ML(aux_M)+(err_sigman2_unk_pilot_ML_num(aux_M)/err_sigman2_unk_pilot_ML_den(aux_M));
                                err_sigman2_unk_pilot_ML_alt(aux_M)=err_sigman2_unk_pilot_ML_alt(aux_M)+(err_sigman2_unk_pilot_ML_num_alt(aux_M)/err_sigman2_unk_pilot_ML_den_alt(aux_M));
                                err_sigman2_unk_pilot_MM(aux_M)=err_sigman2_unk_pilot_MM(aux_M)+(err_sigman2_unk_pilot_MM_num(aux_M)/err_sigman2_unk_pilot_MM_den(aux_M));
                                err_sigman2_mix_pilot_ML(aux_M)=err_sigman2_mix_pilot_ML(aux_M)+(err_sigman2_mix_pilot_ML_num(aux_M)/err_sigman2_mix_pilot_ML_den(aux_M));
                                err_sigman2_mix_pilot_ML_alt(aux_M)=err_sigman2_mix_pilot_ML_alt(aux_M)+(err_sigman2_mix_pilot_ML_num_alt(aux_M)/err_sigman2_mix_pilot_ML_den_alt(aux_M));
                                err_sigman2_mix_pilot_MM(aux_M)=err_sigman2_mix_pilot_MM(aux_M)+(err_sigman2_mix_pilot_MM_num(aux_M)/err_sigman2_mix_pilot_MM_den(aux_M));
                                crlb(aux_M)=crlb(aux_M)+(crlb_unk_num(aux_M)/crlb_unk_den(aux_M));
                                crlb_k(aux_M)=crlb_k(aux_M)+(crlb_k_num(aux_M)/crlb_k_den(aux_M));
                                crlb_unk_pilot(1:M,aux_M)=crlb_unk_pilot(1:M,aux_M)+(crlb_unk_pilot_num(1:M,aux_M)./crlb_unk_pilot_den(1:M,aux_M));
                                crlb_k_unk_pilot(1:M,aux_M)=crlb_k_unk_pilot(1:M,aux_M)+(crlb_k_unk_pilot_num(1:M,aux_M)./crlb_k_unk_pilot_den(1:M,aux_M));
                                crlb_unk_pilot_alt(1:M,aux_M)=crlb_unk_pilot_alt(1:M,aux_M)+(crlb_unk_pilot_num_alt(1:M,aux_M)./crlb_unk_pilot_den_alt(1:M,aux_M));
                                crlb_k_unk_pilot_alt(1:M,aux_M)=crlb_k_unk_pilot_alt(1:M,aux_M)+(crlb_k_unk_pilot_num_alt(1:M,aux_M)./crlb_k_unk_pilot_den_alt(1:M,aux_M));
                                crlb_mix_pilot(1:M,aux_M)=crlb_mix_pilot(1:M,aux_M)+(crlb_mix_pilot_num(1:M,aux_M)./crlb_mix_pilot_den(1:M,aux_M));
                                crlb_k_mix_pilot(1:M,aux_M)=crlb_k_mix_pilot(1:M,aux_M)+(crlb_k_mix_pilot_num(1:M,aux_M)./crlb_k_mix_pilot_den(1:M,aux_M));
                                crlb_mix_pilot_alt(1:M,aux_M)=crlb_mix_pilot_alt(1:M,aux_M)+(crlb_mix_pilot_num_alt(1:M,aux_M)./crlb_mix_pilot_den_alt(1:M,aux_M));
                                crlb_k_mix_pilot_alt(1:M,aux_M)=crlb_k_mix_pilot_alt(1:M,aux_M)+(crlb_k_mix_pilot_num_alt(1:M,aux_M)./crlb_k_mix_pilot_den_alt(1:M,aux_M));
                                
                                err_LS_num(aux_M)=0;
                                err_LS_den(aux_M)=0;
                                err_LS_no_int_num(aux_M)=0;
                                err_LS_no_int_den(aux_M)=0;
                                err_CB_num(aux_M)=0;
                                err_CB_den(aux_M)=0;
                                err_CB_no_int_num(aux_M)=0;
                                err_CB_no_int_den(aux_M)=0;
                                err_CPA_num(aux_M)=0;
                                err_CPA_den(aux_M)=0;
                                err_CPA_noise_ML_num(aux_M)=0;
                                err_CPA_noise_ML_den(aux_M)=0;
                                err_CPA_noise_MM_num(aux_M)=0;
                                err_CPA_noise_MM_den(aux_M)=0;
                                err_CPA_noise_NU_1dB_num(aux_M)=0;
                                err_CPA_noise_NU_1dB_den(aux_M)=0;
                                err_sigman2_ML_num(aux_M)=0;
                                err_sigman2_ML_den(aux_M)=0;
                                err_sigman2_MM_num(aux_M)=0;
                                err_sigman2_MM_den(aux_M)=0;
                                err_sigman2_LS_num(aux_M)=0;
                                err_sigman2_LS_den(aux_M)=0;
                                err_sigman2_LS_PSD_num(aux_M)=0;
                                err_sigman2_LS_PSD_den(aux_M)=0;
                                err_sigman2_unk_pilot_ML_num(aux_M)=0;
                                err_sigman2_unk_pilot_ML_den(aux_M)=0;
                                err_sigman2_unk_pilot_ML_num_alt(aux_M)=0;
                                err_sigman2_unk_pilot_ML_den_alt(aux_M)=0;
                                err_sigman2_unk_pilot_MM_num(aux_M)=0;
                                err_sigman2_unk_pilot_MM_den(aux_M)=0;
                                err_sigman2_mix_pilot_ML_num(aux_M)=0;
                                err_sigman2_mix_pilot_ML_den(aux_M)=0;
                                err_sigman2_mix_pilot_ML_num_alt(aux_M)=0;
                                err_sigman2_mix_pilot_ML_den_alt(aux_M)=0;
                                err_sigman2_mix_pilot_MM_num(aux_M)=0;
                                err_sigman2_mix_pilot_MM_den(aux_M)=0;
                                crlb_unk_num(aux_M)=0;
                                crlb_unk_den(aux_M)=0;
                                crlb_k_num(aux_M)=0;
                                crlb_k_den(aux_M)=0;
                                crlb_unk_pilot_num(1:M,aux_M)=zeros(M,1);
                                crlb_unk_pilot_den(1:M,aux_M)=zeros(M,1);
                                crlb_k_unk_pilot_num(1:M,aux_M)=zeros(M,1);
                                crlb_k_unk_pilot_den(1:M,aux_M)=zeros(M,1);
                                crlb_unk_pilot_num_alt(1:M,aux_M)=zeros(M,1);
                                crlb_unk_pilot_den_alt(1:M,aux_M)=zeros(M,1);
                                crlb_k_unk_pilot_num_alt(1:M,aux_M)=zeros(M,1);
                                crlb_k_unk_pilot_den_alt(1:M,aux_M)=zeros(M,1);
                                crlb_mix_pilot_num(1:M,aux_M)=zeros(M,1);
                                crlb_mix_pilot_den(1:M,aux_M)=zeros(M,1);
                                crlb_k_mix_pilot_num(1:M,aux_M)=zeros(M,1);
                                crlb_k_mix_pilot_den(1:M,aux_M)=zeros(M,1);
                                crlb_mix_pilot_num_alt(1:M,aux_M)=zeros(M,1);
                                crlb_mix_pilot_den_alt(1:M,aux_M)=zeros(M,1);
                                crlb_k_mix_pilot_num_alt(1:M,aux_M)=zeros(M,1);
                                crlb_k_mix_pilot_den_alt(1:M,aux_M)=zeros(M,1);
                            end
                            
                            if rate_vs_ant_num==1
                                
                                %%%% Per-cell Rate, Number of antennas %%%%
                                
                            end
                        end
                        
                        if error_vs_std_dev_AOA==1
                            
                            if error_vs_std_dev_AOA==1
                                
                                %%%% Estimation Error, Standard Deviation of AOA %%%%
                                err_LS(aux_sigma_AOA)=err_LS(aux_sigma_AOA)+(err_LS_num(aux_sigma_AOA)/err_LS_den(aux_sigma_AOA));
                                err_LS_no_int(aux_sigma_AOA)=err_LS_no_int(aux_sigma_AOA)+(err_LS_no_int_num(aux_sigma_AOA)/err_LS_no_int_den(aux_sigma_AOA));
                                err_CB(aux_sigma_AOA)=err_CB(aux_sigma_AOA)+(err_CB_num(aux_sigma_AOA)/err_CB_den(aux_sigma_AOA));
                                err_CB_no_int(aux_sigma_AOA)=err_CB_no_int(aux_sigma_AOA)+(err_CB_no_int_num(aux_sigma_AOA)/err_CB_no_int_den(aux_sigma_AOA));
                                err_CPA(aux_sigma_AOA)=err_CPA(aux_sigma_AOA)+(err_CPA_num(aux_sigma_AOA)/err_CPA_den(aux_sigma_AOA));
                                err_CPA_noise_ML(aux_sigma_AOA)=err_CPA_noise_ML(aux_sigma_AOA)+(err_CPA_noise_ML_num(aux_sigma_AOA)/err_CPA_noise_ML_den(aux_sigma_AOA));
                                err_CPA_noise_MM(aux_sigma_AOA)=err_CPA_noise_MM(aux_sigma_AOA)+(err_CPA_noise_MM_num(aux_sigma_AOA)/err_CPA_noise_MM_den(aux_sigma_AOA));
                                err_CPA_noise_NU_1dB(aux_sigma_AOA)=err_CPA_noise_NU_1dB(aux_sigma_AOA)+(err_CPA_noise_NU_1dB_num(aux_sigma_AOA)/err_CPA_noise_NU_1dB_den(aux_sigma_AOA));
                                
                                err_sigman2_ML(aux_sigma_AOA)=err_sigman2_ML(aux_sigma_AOA)+(err_sigman2_ML_num(aux_sigma_AOA)/err_sigman2_ML_den(aux_sigma_AOA));
                                err_sigman2_MM(aux_sigma_AOA)=err_sigman2_MM(aux_sigma_AOA)+(err_sigman2_MM_num(aux_sigma_AOA)/err_sigman2_MM_den(aux_sigma_AOA));
                                err_sigman2_unk_pilot_ML(aux_sigma_AOA)=err_sigman2_unk_pilot_ML(aux_sigma_AOA)+(err_sigman2_unk_pilot_ML_num(aux_sigma_AOA)/err_sigman2_unk_pilot_ML_den(aux_sigma_AOA));
                                err_sigman2_unk_pilot_ML_alt(aux_sigma_AOA)=err_sigman2_unk_pilot_ML_alt(aux_sigma_AOA)+(err_sigman2_unk_pilot_ML_num_alt(aux_sigma_AOA)/err_sigman2_unk_pilot_ML_den_alt(aux_sigma_AOA));
                                err_sigman2_unk_pilot_MM(aux_sigma_AOA)=err_sigman2_unk_pilot_MM(aux_sigma_AOA)+(err_sigman2_unk_pilot_MM_num(aux_sigma_AOA)/err_sigman2_unk_pilot_MM_den(aux_sigma_AOA));
                                err_sigman2_mix_pilot_ML(aux_sigma_AOA)=err_sigman2_mix_pilot_ML(aux_sigma_AOA)+(err_sigman2_mix_pilot_ML_num(aux_sigma_AOA)/err_sigman2_mix_pilot_ML_den(aux_sigma_AOA));
                                err_sigman2_mix_pilot_ML_alt(aux_sigma_AOA)=err_sigman2_mix_pilot_ML_alt(aux_sigma_AOA)+(err_sigman2_mix_pilot_ML_num_alt(aux_sigma_AOA)/err_sigman2_mix_pilot_ML_den_alt(aux_sigma_AOA));
                                err_sigman2_mix_pilot_MM(aux_sigma_AOA)=err_sigman2_mix_pilot_MM(aux_sigma_AOA)+(err_sigman2_mix_pilot_MM_num(aux_sigma_AOA)/err_sigman2_mix_pilot_MM_den(aux_sigma_AOA));
                                crlb(aux_sigma_AOA)=crlb(aux_sigma_AOA)+(crlb_unk_num(aux_sigma_AOA)/crlb_unk_den(aux_sigma_AOA));
                                crlb_k(aux_sigma_AOA)=crlb_k(aux_sigma_AOA)+(crlb_k_num(aux_sigma_AOA)/crlb_k_den(aux_sigma_AOA));
                                crlb_unk_pilot(1:M,aux_sigma_AOA)=crlb_unk_pilot(1:M,aux_sigma_AOA)+(crlb_unk_pilot_num(1:M,aux_sigma_AOA)./crlb_unk_pilot_den(1:M,aux_sigma_AOA));
                                crlb_k_unk_pilot(1:M,aux_sigma_AOA)=crlb_k_unk_pilot(1:M,aux_sigma_AOA)+(crlb_k_unk_pilot_num(1:M,aux_sigma_AOA)./crlb_k_unk_pilot_den(1:M,aux_sigma_AOA));
                                crlb_unk_pilot_alt(1:M,aux_sigma_AOA)=crlb_unk_pilot_alt(1:M,aux_sigma_AOA)+(crlb_unk_pilot_num_alt(1:M,aux_sigma_AOA)./crlb_unk_pilot_den_alt(1:M,aux_sigma_AOA));
                                crlb_k_unk_pilot_alt(1:M,aux_sigma_AOA)=crlb_k_unk_pilot_alt(1:M,aux_sigma_AOA)+(crlb_k_unk_pilot_num_alt(1:M,aux_sigma_AOA)./crlb_k_unk_pilot_den_alt(1:M,aux_sigma_AOA));
                                crlb_mix_pilot(1:M,aux_sigma_AOA)=crlb_mix_pilot(1:M,aux_sigma_AOA)+(crlb_mix_pilot_num(1:M,aux_sigma_AOA)./crlb_mix_pilot_den(1:M,aux_sigma_AOA));
                                crlb_k_mix_pilot(1:M,aux_sigma_AOA)=crlb_k_mix_pilot(1:M,aux_sigma_AOA)+(crlb_k_mix_pilot_num(1:M,aux_sigma_AOA)./crlb_k_mix_pilot_den(1:M,aux_sigma_AOA));
                                crlb_mix_pilot_alt(1:M,aux_sigma_AOA)=crlb_mix_pilot_alt(1:M,aux_sigma_AOA)+(crlb_mix_pilot_num_alt(1:M,aux_sigma_AOA)./crlb_mix_pilot_den_alt(1:M,aux_sigma_AOA));
                                crlb_k_mix_pilot_alt(1:M,aux_sigma_AOA)=crlb_k_mix_pilot_alt(1:M,aux_sigma_AOA)+(crlb_k_mix_pilot_num_alt(1:M,aux_sigma_AOA)./crlb_k_mix_pilot_den_alt(1:M,aux_sigma_AOA));
                                
                                err_LS_num(aux_sigma_AOA)=0;
                                err_LS_den(aux_sigma_AOA)=0;
                                err_LS_no_int_num(aux_sigma_AOA)=0;
                                err_LS_no_int_den(aux_sigma_AOA)=0;
                                err_CB_num(aux_sigma_AOA)=0;
                                err_CB_den(aux_sigma_AOA)=0;
                                err_CB_no_int_num(aux_sigma_AOA)=0;
                                err_CB_no_int_den(aux_sigma_AOA)=0;
                                err_CPA_num(aux_sigma_AOA)=0;
                                err_CPA_den(aux_sigma_AOA)=0;
                                err_CPA_noise_ML_num(aux_sigma_AOA)=0;
                                err_CPA_noise_ML_den(aux_sigma_AOA)=0;
                                err_CPA_noise_MM_num(aux_sigma_AOA)=0;
                                err_CPA_noise_MM_den(aux_sigma_AOA)=0;
                                err_CPA_noise_NU_1dB_num(aux_sigma_AOA)=0;
                                err_CPA_noise_NU_1dB_den(aux_sigma_AOA)=0;
                                err_sigman2_ML_num(aux_sigma_AOA)=0;
                                err_sigman2_ML_den(aux_sigma_AOA)=0;
                                err_sigman2_MM_num(aux_sigma_AOA)=0;
                                err_sigman2_MM_den(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_ML_num(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_ML_den(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_ML_num_alt(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_ML_den_alt(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_MM_num(aux_sigma_AOA)=0;
                                err_sigman2_unk_pilot_MM_den(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_ML_num(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_ML_den(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_ML_num_alt(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_ML_den_alt(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_MM_num(aux_sigma_AOA)=0;
                                err_sigman2_mix_pilot_MM_den(aux_sigma_AOA)=0;
                                crlb_unk_num(aux_sigma_AOA)=0;
                                crlb_unk_den(aux_sigma_AOA)=0;
                                crlb_k_num(aux_sigma_AOA)=0;
                                crlb_k_den(aux_sigma_AOA)=0;
                                crlb_unk_pilot_num(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_unk_pilot_den(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_unk_pilot_num(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_unk_pilot_den(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_unk_pilot_num_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_unk_pilot_den_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_unk_pilot_num_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_unk_pilot_den_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_mix_pilot_num(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_mix_pilot_den(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_mix_pilot_num(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_mix_pilot_den(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_mix_pilot_num_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_mix_pilot_den_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_mix_pilot_num_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                                crlb_k_mix_pilot_den_alt(1:M,aux_sigma_AOA)=zeros(M,1);
                            end
                            
                            if rate_vs_std_dev_AOA==1
                                
                                %%%% Per-cell Rate, Standard Deviation of AOA %%%%
                                
                            end
                        end
                        
                        if comp_cap==1
                            
                            W_BF=zeros(M,U,L);
                            W_RZF=zeros(M,U,L);
                            for l=1:L
                                W_BF(:,:,l)=H_est(:,:,l);
                                W_RZF(:,:,l)=(H_est(:,:,l)*H_est(:,:,l)'+Z_dl+M*phi_dl*eye(M,M))\H_est(:,:,l);
                            end
                            
                            J=1;
                            
                            for l=1:L
                                lambda_BF_aux(MC_aux,l,aux_M,1)=((1/U)*trace(W_BF(:,:,l)*W_BF(:,:,l)'));
                                lambda_RZF_aux(MC_aux,l,aux_M,1)=((1/U)*trace(W_RZF(:,:,l)*W_RZF(:,:,l)'));
                            end
                            
                            for K=1:U
                                
                                R_BF_empirical_P_S(MC_aux,aux_M,K,1)=(H(:,J,J,K)'*W_BF(:,K,J));
                                R_RZF_empirical_P_S(MC_aux,aux_M,K,1)=(H(:,J,J,K)'*W_RZF(:,K,J));
                                for l=1:L
                                    for k=1:U
                                        R_BF_empirical_P_I2(MC_aux,aux_M,K,l,k,1)=(abs(H(:,l,J,K)'*W_BF(:,k,l))^2);
                                        R_RZF_empirical_P_I2(MC_aux,aux_M,K,l,k,1)=(abs(H(:,l,J,K)'*W_RZF(:,k,l))^2);
                                    end
                                end
                            end
                        end
                        
                    end
                end
            end
        end
        
        if comp_sim==1
            %% Results 
            SNR_sim=10*log10(S_sim./N_sim);
            
            if error_vs_ant_num==1 || error_vs_std_dev_AOA==1
                
                %%%% Estimation Error %%%%
                err_LS=10*log10(err_LS/(1*MC_rep));
                err_LS_no_int=10*log10(err_LS_no_int/(1*MC_rep));
                err_CB=10*log10(err_CB/(1*MC_rep));
                err_CB_no_int=10*log10(err_CB_no_int/(1*MC_rep));
                
                err_CPA=(err_CPA/(1*MC_rep));
                err_CPA_noise_ML=(err_CPA_noise_ML/(1*MC_rep));
                err_CPA_noise_MM=(err_CPA_noise_MM/(1*MC_rep));
                err_CPA_noise_NU_1dB=(err_CPA_noise_NU_1dB/(1*MC_rep));
                
                err_sigman2_ML=10*log10(err_sigman2_ML/(1*MC_rep));
                err_sigman2_MM=10*log10(err_sigman2_MM/(1*MC_rep));
                err_sigman2_LS=10*log10(err_sigman2_LS/(1*MC_rep));
                err_sigman2_LS_PSD=10*log10(err_sigman2_LS_PSD/(1*MC_rep));
                err_sigman2_unk_pilot_ML=10*log10(err_sigman2_unk_pilot_ML/(1*MC_rep));
                err_sigman2_unk_pilot_ML_alt=10*log10(err_sigman2_unk_pilot_ML_alt/(1*MC_rep));
                err_sigman2_unk_pilot_MM=10*log10(err_sigman2_unk_pilot_MM/(1*MC_rep));
                err_sigman2_mix_pilot_ML=10*log10(err_sigman2_mix_pilot_ML/(1*MC_rep));
                err_sigman2_mix_pilot_ML_alt=10*log10(err_sigman2_mix_pilot_ML_alt/(1*MC_rep));
                err_sigman2_mix_pilot_MM=10*log10(err_sigman2_mix_pilot_MM/(1*MC_rep));
                crlb=10*log10(crlb/(1*MC_rep));
                crlb_k=10*log10(crlb_k/(1*MC_rep));
                crlb_unk_pilot_final=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_unk_pilot_final(m1)=crlb_unk_pilot_final(m1)+(1/(-(crlb_unk_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_unk_pilot_final(m1)=crlb_unk_pilot_final(m1)+(1/(-(crlb_unk_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_unk_pilot_final=10*log10(crlb_unk_pilot_final./sigman2_sum_crlb);
                crlb_k_unk_pilot_final=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_k_unk_pilot_final(m1)=crlb_k_unk_pilot_final(m1)+(1/(-(crlb_k_unk_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_k_unk_pilot_final(m1)=crlb_k_unk_pilot_final(m1)+(1/(-(crlb_k_unk_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_k_unk_pilot_final=10*log10(crlb_k_unk_pilot_final./sigman2_sum_crlb);
                
                crlb_unk_pilot_final_alt=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_unk_pilot_final_alt(m1)=crlb_unk_pilot_final_alt(m1)+(1/(-(crlb_unk_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_unk_pilot_final_alt(m1)=crlb_unk_pilot_final_alt(m1)+(1/(-(crlb_unk_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_unk_pilot_final_alt=10*log10(crlb_unk_pilot_final_alt./sigman2_sum_crlb);
                crlb_k_unk_pilot_final_alt=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_k_unk_pilot_final_alt(m1)=crlb_k_unk_pilot_final_alt(m1)+(1/(-(crlb_k_unk_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_k_unk_pilot_final_alt(m1)=crlb_k_unk_pilot_final_alt(m1)+(1/(-(crlb_k_unk_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_k_unk_pilot_final_alt=10*log10(crlb_k_unk_pilot_final_alt./sigman2_sum_crlb);
                
                crlb_mix_pilot_final=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_mix_pilot_final(m1)=crlb_mix_pilot_final(m1)+(1/(-(crlb_mix_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_mix_pilot_final(m1)=crlb_mix_pilot_final(m1)+(1/(-(crlb_mix_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_mix_pilot_final=10*log10(crlb_mix_pilot_final./sigman2_sum_crlb);
                crlb_k_mix_pilot_final=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_k_mix_pilot_final(m1)=crlb_k_mix_pilot_final(m1)+(1/(-(crlb_k_mix_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_k_mix_pilot_final(m1)=crlb_k_mix_pilot_final(m1)+(1/(-(crlb_k_mix_pilot(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_k_mix_pilot_final=10*log10(crlb_k_mix_pilot_final./sigman2_sum_crlb);
                
                crlb_mix_pilot_final_alt=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_mix_pilot_final_alt(m1)=crlb_mix_pilot_final_alt(m1)+(1/(-(crlb_mix_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_mix_pilot_final_alt(m1)=crlb_mix_pilot_final_alt(m1)+(1/(-(crlb_mix_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_mix_pilot_final_alt=10*log10(crlb_mix_pilot_final_alt./sigman2_sum_crlb);
                crlb_k_mix_pilot_final_alt=zeros(1,M_max);
                if error_vs_ant_num==1
                    for m1=1:M_max
                        for m2=1:M_ini(m1)
                            crlb_k_mix_pilot_final_alt(m1)=crlb_k_mix_pilot_final_alt(m1)+(1/(-(crlb_k_mix_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                else
                    for m1=1:deviation_max
                        for m2=1:M
                            crlb_k_mix_pilot_final_alt(m1)=crlb_k_mix_pilot_final_alt(m1)+(1/(-(crlb_k_mix_pilot_alt(m2,m1)/(1*MC_rep))));
                        end
                    end
                end
                crlb_k_mix_pilot_final_alt=10*log10(crlb_k_mix_pilot_final_alt./sigman2_sum_crlb);
            end
            
            if rate_vs_ant_num==1
                
                %%%% Per-cell Rate %%%%
                for aux_M=1:length(M_plot)
                    lambda_BF=zeros(L,1);
                    lambda_RZF=zeros(L,1);
                    for l=1:L
                        lambda_BF(l)=(1)/(mean(lambda_BF_aux(:,l,aux_M,1)));
                        lambda_RZF(l)=(1)/(mean(lambda_RZF_aux(:,l,aux_M,1)));
                    end
                    
                    for K=1:U
                        sum_BF=0;
                        sum_RZF=0;
                        for l=1:L
                            for k=1:U
                                if l==J && k==K
                                else
                                    sum_BF=sum_BF+lambda_BF(l)*mean(R_BF_empirical_P_I2(:,aux_M,K,l,k,1));
                                    sum_RZF=sum_RZF+lambda_RZF(l)*mean(R_RZF_empirical_P_I2(:,aux_M,K,l,k,1));
                                end
                            end
                        end
                        SINR=(lambda_BF(J)*abs(mean(R_BF_empirical_P_S(:,aux_M,K,1)))^2)/((1/rho) + lambda_BF(J)*var(R_BF_empirical_P_S(:,aux_M,K,1))+sum_BF);
                        R_BF_empirical_coor_user(K,aux_M)=R_BF_empirical_coor_user(K,aux_M)+log2(1+SINR);
                        
                        SINR=(lambda_RZF(J)*abs(mean(R_RZF_empirical_P_S(:,aux_M,K,1)))^2)/((1/rho) + lambda_RZF(J)*var(R_RZF_empirical_P_S(:,aux_M,K,1))+sum_RZF);
                        N_SINR_emp(K,aux_M)=(1/rho);
                        S_SINR_emp(K,aux_M)=lambda_RZF(J)*abs(mean(R_RZF_empirical_P_S(:,aux_M,K,1)))^2;
                        Sp_SINR_emp(K,aux_M)=lambda_RZF(J)*var(R_RZF_empirical_P_S(:,aux_M,K,1));
                        I_SINR_emp(K,aux_M)=sum_RZF;
                        R_RZF_empirical_coor_user(K,aux_M)=R_RZF_empirical_coor_user(K,aux_M)+log2(1+SINR);
                    end
                end
                R_BF_empirical_coor_user=R_BF_empirical_coor_user/(1);
                R_RZF_empirical_coor_user=R_RZF_empirical_coor_user/(1);
                
                for M_aux=1:length(M_plot)
                    R_BF_empirical(M_aux)=mean(R_BF_empirical_coor_user(:,M_aux));
                    R_RZF_empirical(M_aux)=mean(R_RZF_empirical_coor_user(:,M_aux));
                    
                    N_SINR_mean_emp(M_aux)=mean(N_SINR_emp(:,M_aux));
                    S_SINR_mean_emp(M_aux)=mean(S_SINR_emp(:,M_aux));
                    Sp_SINR_mean_emp(M_aux)=mean(Sp_SINR_emp(:,M_aux));
                    I_SINR_mean_emp(M_aux)=mean(I_SINR_emp(:,M_aux));
                end
                
            end
                 
        end
        
        if comp_cap==1
            
            R_BF_theoretical_coor_user=R_BF_theoretical_coor_user/(1);
            R_RZF_theoretical_coor_user=R_RZF_theoretical_coor_user/(1);
            
            for M_aux=1:length(M_plot)
                R_BF_theoretical(M_aux)=mean(R_BF_theoretical_coor_user(:,M_aux));
                R_RZF_theoretical(M_aux)=mean(R_RZF_theoretical_coor_user(:,M_aux));
            end
            
            for M_aux=1:length(M_plot)
                
                N_SINR_mean(M_aux)=mean(N_SINR(:,M_aux));
                S_SINR_mean(M_aux)=mean(S_SINR(:,M_aux));
                Sp_SINR_mean(M_aux)=mean(Sp_SINR(:,M_aux));
                I_SINR_mean(M_aux)=mean(I_SINR(:,M_aux));
            end
            
        end
        
        if noise_var_est==1
            sweep(1,:,sweep_aux)=err_sigman2_ML;
            sweep(2,:,sweep_aux)=err_sigman2_MM;
            sweep(3,:,sweep_aux)=err_sigman2_unk_pilot_ML;
            sweep(4,:,sweep_aux)=err_sigman2_unk_pilot_ML_alt;
            sweep(5,:,sweep_aux)=err_sigman2_unk_pilot_MM;
            sweep(6,:,sweep_aux)=err_sigman2_mix_pilot_ML;
            sweep(7,:,sweep_aux)=err_sigman2_mix_pilot_ML_alt;
            sweep(8,:,sweep_aux)=err_sigman2_mix_pilot_MM;
            sweep(9,:,sweep_aux)=crlb;
            sweep(10,:,sweep_aux)=crlb_unk_pilot_final;
            sweep(11,:,sweep_aux)=crlb_mix_pilot_final;
            sweep(12,:,sweep_aux)=crlb_unk_pilot_final_alt;
            sweep(13,:,sweep_aux)=crlb_mix_pilot_final_alt;
            sweep(14,:,sweep_aux)=err_CPA;
            sweep(15,:,sweep_aux)=err_CPA_noise_ML;
            sweep(16,:,sweep_aux)=err_CPA_noise_MM;
            sweep(17,:,sweep_aux)=err_CPA_noise_NU_1dB;
            sweep(18,:,sweep_aux)=crlb_k_mix_pilot_final_alt;
            sweep(19,:,sweep_aux)=err_sigman2_LS;
            sweep(20,:,sweep_aux)=err_sigman2_LS_PSD;
            
        end
        if comp_cap==1
            sweep(1,:,sweep_aux)=R_RZF_theoretical;
            sweep(2,:,sweep_aux)=R_BF_theoretical;
            sweep(3,:,sweep_aux)=R_BF_empirical;
            sweep(4,:,sweep_aux)=R_RZF_empirical;
            sweep(5,:,sweep_aux)=rank_R(1,:); % rank
            sweep(6,:,sweep_aux)=rank_R(2,:); % cond
            sweep(7,:,sweep_aux)=mse_R(1,:); % mse
            sweep(8,:,sweep_aux)=mean_AOA_gen(1,1,2,1)-sigma_theta_AOA<=mean_AOA_gen(1,1,1,1)+sigma_theta_AOA;
            % mse
            sweep(14,:,sweep_aux)=mean_AOA_gen(1,1,2,2)+sigma_theta_AOA>=mean_AOA_gen(1,1,1,2)+2*pi-sigma_theta_AOA;
            
            sweep(9,:,sweep_aux)=N_SINR_mean;
            sweep(10,:,sweep_aux)=S_SINR_mean;
            sweep(11,:,sweep_aux)=Sp_SINR_mean;
            sweep(12,:,sweep_aux)=I_SINR_mean;
            sweep(13,:,sweep_aux)=(S_SINR_mean)/(N_SINR_mean+Sp_SINR_mean+I_SINR_mean);
            
            sweep(15,:,sweep_aux)=N_SINR_mean_emp;
            sweep(16,:,sweep_aux)=S_SINR_mean_emp;
            sweep(17,:,sweep_aux)=Sp_SINR_mean_emp;
            sweep(18,:,sweep_aux)=I_SINR_mean_emp;
            sweep(19,:,sweep_aux)=(S_SINR_mean_emp)/(N_SINR_mean_emp+Sp_SINR_mean_emp+I_SINR_mean_emp);
        end
        save sweep_channel sweep
        sweep_aux=sweep_aux+1;
    end
    sweep_f=sweep_f+sweep;
end
sweep_f=sweep_f/cell_edge_dist_rep;
